/**
 * \file gre/lrscript.h
 * \brief GRE::LAYER_SCRIPT definitions
 *
 * \if NODOC
 * $Id: lrscript.h_v 1.9 2013/02/15 20:51:39 mju Exp $
 *
 * $Log: lrscript.h_v $
 * Revision 1.9  2013/02/15 20:51:39  mju
 * Fix dox hdr.
 *
 * Revision 1.8  2006/09/28 19:56:25  mju
 * Use rvc objitem hdr.
 *
 * Revision 1.7  2006/09/15 15:30:52  mju
 * Serializer changes.
 *
 * Revision 1.6  2006/09/07 16:31:43  mju
 * Use rvc classes.
 *
 * Revision 1.5  2005/10/19 20:34:24  mju
 * Use gre namespace instead of legacy names.
 *
 * Revision 1.4  2005/10/14 17:31:54  mju
 * GRE namespace and treeview.
 *
 * Revision 1.3  2005/03/31 16:57:06  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.2  2005/02/28 18:08:20  scowan
 * Support layer copy virtual method.
 *
 * Revision 1.1  2005/01/20 16:57:29  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LRSCRIPT_H
#define	INC_GRE_LRSCRIPT_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif

namespace GRE {
//===================================================================================================================

class LAYER_SCRIPT : public GRE::LAYER {
	public:

		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;

				RVC::OBJITEM m_ObjItemScript;				//!< Location of script, may refer to file if script in text file
				RVC::OBJITEM m_ObjItemStyle;				//!< Location of style object if used
				MISTRING m_Script;							//!< Script string
				DRECT2D m_Extents;							//!< Extents in Page Units relative to MapScale
				int m_PageUnitsCode;
				int m_MapUnitsCode;
				int m_LatLonFmtCode;
				int m_ExtentsMode;							//!< 0=Auto, 1=Manual, 2=MatchGroup
				double m_MapScale;							//!< Map Scale above extents are relative to (if not layout scale)
				double m_Margin;								//!< Computed by script, not serialized
				UINT8 m_UsePageUnits;						//!< True if above extents are page units.  Same as Region.Extents if false
				SPATREF::COORDREFSYS m_CoordRefSys;

				DISPPARM ();
				DISPPARM (
					const DISPPARM& rhs
					): GRE::LAYER::DISPPARM(rhs) { Copy(rhs); }
				virtual ~DISPPARM (
					) { Free(); }

				DISPPARM& operator= (const DISPPARM& rhs) {
					if (this != &rhs) {
						Free();
						Copy(rhs);
						}
					return (*this);
					}

				ERRVALUE ComputeExtents (LAYER_SCRIPT*);

			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				void Copy (const DISPPARM& rhs);
				void Free ();
				friend class LAYER_SCRIPT;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Register this layer type
		static void RegisterType (
			);

		//! Construct layer and insert into list
		explicit LAYER_SCRIPT (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_SCRIPT ();

		double ComputeScale (
			) const;

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm (
			) const {
			return (m_dispparm);
			}

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// Overrides from GRE::LAYER.
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetDftName (bool ReplaceExisting);
		virtual int v_UpdateExtents (bool forceupdate);
		virtual void v_ComputeMargins(double& tmargin, double&bmargin, double&lmargin, double&rmargin);

		LAYER_SCRIPT (const LAYER_SCRIPT&);
		LAYER_SCRIPT& operator= (const LAYER_SCRIPT&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================
}	// End namespace GRE

#endif	// INC_GRE_LRSCRIPT_H

