/**
 * \file gre/uiformautolabel.h
 * \brief GRE::UI::FORM_AUTOLABEL definitions
 *
 * \if NODOC
 * $Id: uiformautolabel.h_v 1.3 2012/05/25 21:05:46 mju Exp $
 *
 * $Log: uiformautolabel.h_v $
 * Revision 1.3  2012/05/25 21:05:46  mju
 * Add options for point/line/area placement.
 *
 * Revision 1.1  2012/05/10 17:09:18  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_UIFORMAUTOLABEL_H
#define	INC_GRE_UIFORMAUTOLABEL_H

#ifndef	INC_GRE_ELEMPARMS_H
#include <gre/elemparms.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MGUI_EDIT_H
#include <mgui/edit.h>
#endif

#ifndef	INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif

// Forward declarations
#ifndef	GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class DBASE_ELEMENT;
	}
#endif

namespace GRE {
namespace UI {

//! Form for setting automatic label parameters (GRE::AUTOLABELPARMS).
class FORM_AUTOLABEL : public MGUI::FORM_COMPOSITE {
	public:

		//!< Options for form creation.
		enum OPTIONS {
			OPTIONS_None =				0x00,
			OPTION_PlacePoint =		0x01,		//!< Include "point/position" controls
			OPTION_PlaceLine =		0x02,		//!< Include "line/position" controls
			OPTION_PlaceArea =		0x04,		//!< Include "area/position" controls
			OPTION_PlaceAll =			OPTION_PlacePoint | OPTION_PlaceLine | OPTION_PlaceArea,
			OPTION_Optimization =	0x08,		//!< Include "optimization" controls
			OPTION_Priority =			0x10,		//!< Include "priority" controls
			OPTION_Narrow =			0x20,		//!< Use narrower layout
			};

		//! Constructor.
		FORM_AUTOLABEL ();

		//! Destructor.
		virtual ~FORM_AUTOLABEL ();

		//! Add item to "area/place" combobox.
		void AddCbxItemPlaceArea (
			AUTOLABELPARMS::PLACE_AREA PlaceArea,
			TEXTID textid = TEXTID__None				//!< Label, TEXTID__None for default
			);

		//! Add item to "line/place" combobox.
		void AddCbxItemPlaceLine (
			AUTOLABELPARMS::PLACE_LINE PlaceLine,
			TEXTID textid = TEXTID__None				//!< Label, TEXTID__None for default
			);

		//! Add item to "point/place" combobox.
		void AddCbxItemPlacePoint (
			AUTOLABELPARMS::PLACE_POINT PlacePoint,
			TEXTID textid = TEXTID__None				//!< Label, TEXTID__None for default
			);

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& paneParent,	//!< Parent pane
			const RVC::OBJITEM& ObjItem,			//!< Location of geometric object, may be unassigned
			ELEMTYPE ElemType,						//!< Geometric element type
			OPTIONS options = OPTIONS_None		//!< Form options
			);

		//! Get current values.
		void GetValues (
			AUTOLABELPARMS& values					//!< Container for values returned, only those managed by form are updated
			);

		//! Set delegate for getting current map scale.
		void SetDelegateGetCurMapScale (
			DELEGATE_DOUBLE_NOPARMS delegate		//!< Delegate to set
			) { m_DelegateGetCurMapScale = delegate; }

		//! Set delegate for getting OBJITEM for style.
		void SetDelegateGetObjItemStyle (
			DELEGATE_OBJITEM_NOPARMS delegate	//!< Delegate to set
			) { m_DelegateGetObjItemStyle = delegate; }

		//! Set delegate to call when value changed.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate		//!< Delegate to set
			) { m_DelegateOnChange = delegate; }

		//! Set associated geometric object location.
		void SetObjItem (
			const RVC::OBJITEM& ObjItem			//!< Location of geometric object
			);

		//! Set values.
		void SetValues (
			const AUTOLABELPARMS& values,			//!< Values to set
			bool notify									//!< Whether to notify of change via delegate
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_AUTOLABEL THISCLASS;

		RVC::OBJITEM m_ObjItem;
		mutable RVC::OBJITEM m_ObjItemStyle;
		RVC::DBASE_ELEMENT *m_pObjDb;
		ELEMTYPE m_ElemType;
		AUTOLABELPARMS m_values;
		DELEGATE_DOUBLE_NOPARMS m_DelegateGetCurMapScale;
		DELEGATE_OBJITEM_NOPARMS m_DelegateGetObjItemStyle;
		DELEGATE_VOID_NOPARMS m_DelegateOnChange;
		MGUI::FORM_COMBOBOX m_CbxText;
		MGUI::CTRL_PUSHBUTTON m_BtnTextSpecify;
		MGUI::CTRL_EDIT_STRING m_TxtTextField;
		MGUI::FORM_EDIT_RANGE m_FormScaleRange;
		MGUI::FORM_COMBOBOX m_CbxPlacePoint;
		MGUI::FORM_COMBOBOX m_CbxPlaceLine;
		MGUI::FORM_COMBOBOX m_CbxPlaceArea;
		MGUI::CTRL_PUSHBUTTON m_BtnTextStyle;
		MGUI::CTRL_PUSHBUTTON m_BtnFrameStyle;
		MGUI::FORM_COMBOBOX m_CbxOptimization;
		MGUI::FORM_COMBOBOX m_CbxPriority;
		MGUI::CTRL_PUSHBUTTON m_BtnPrioritySpecify;
		MGUI::CTRL_EDIT_STRING m_TxtPriorityField;

		ERRVALUE DisplayTableField (MGUI::CTRL_EDIT_STRING& ctrl, const RVC::OBJECTNAME& TableName, const RVC::OBJECTNAME& FieldName);
		ERRVALUE EditScript ();
		void EnableControls ();
		double GetCurMapScale () const;
		const RVC::OBJITEM& GetObjItemStyle () const;
		void NotifyChange ();

		void OnCbxOptimization ();
		void OnCbxPriority ();
		void OnCbxText ();
		void OnCmdFrameStyle ();
		void OnCmdPrioritySpecify ();
		void OnCmdTextSpecify ();
		void OnCmdTextStyle ();

		ERRVALUE SelectTableField (RVC::OBJECTNAME& TableName, RVC::OBJECTNAME& FieldName, MGUI::CTRL_EDIT_STRING& TextCtrl, TEXTID DlgTitle = TEXTID__None);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_AUTOLABEL::OPTIONS)
#endif

}	// End namespace UI
}	// End namespace GRE

#endif	// INC_GRE_UIFORMAUTOLABEL_H
