/**
 * \file mgui/dlggetobjectdlg.h
 * \brief Dialog classes to select files/objects
 *
 * \if NODOC
 * $Id: dlggetobjectdlg.h_v 1.76 2013/01/22 23:22:58 bcolombe Exp $
 *
 * $Log: dlggetobjectdlg.h_v $
 * Revision 1.76  2013/01/22 23:22:58  bcolombe
 * Add v_AddedItemsEnd
 *
 * Revision 1.75  2012/12/17 21:27:56  bcolombe
 * Add success,failure,reliability fields
 *
 * Revision 1.74  2012/11/19 15:58:03  bcolombe
 * Add CanSelectItemAsObject
 *
 * Revision 1.73  2012/11/06 20:33:48  bcolombe
 * Add onFavorites,RecnetActive
 *
 * Revision 1.72  2012/10/02 14:41:11  bcolombe
 * Change OnNavigateSelectItem to return bool
 *
 * Revision 1.71  2012/09/12 22:41:44  bcolombe
 * Added paste and go button
 *
 * Revision 1.70  2012/09/11 16:12:06  bcolombe
 * Support fusion table.
 *
 * Revision 1.69  2012/07/20 20:54:41  bcolombe
 * Add cmdids
 *
 * Revision 1.68  2012/04/03 22:06:34  bcolombe
 * Fix for showing download err
 *
 * Revision 1.67  2012/03/05 18:41:21  bcolombe
 * Changes for pattern matching when adding all
 *
 * Revision 1.66  2012/02/06 15:41:39  bcolombe
 * Add IsNodeBrowsable
 *
 * Revision 1.65  2012/02/02 20:31:50  bcolombe
 * Use TILESET::LIST for reading webtilesets xml
 *
 * Revision 1.64  2011/11/08 22:26:15  mju
 * Remove redundant hdrs.
 *
 * Revision 1.63  2011/09/19 14:31:27  bcolombe
 * Add overwrite message
 *
 * Revision 1.62  2011/08/26 12:36:08  mju
 * Use formradiobuttons hdr.
 *
 * Revision 1.61  2011/04/20 20:59:49  bcolombe
 * Remove OnEditNewItemGainFocus
 *
 * Revision 1.60  2011/03/14 18:39:12  bcolombe
 * Add BuildBrowseListGeomashup
 *
 * Revision 1.59  2011/01/28 00:23:48  bcolombe
 * Remove web location
 *
 * Revision 1.58  2011/01/25 17:42:17  bcolombe
 * Add CreateFile button
 *
 * Revision 1.57  2011/01/19 23:47:50  bcolombe
 * Add prevobjitem for set
 *
 * Revision 1.56  2011/01/19 20:58:27  bcolombe
 * Add m_ErrorDownloading member
 *
 * Revision 1.55  2011/01/17 19:46:17  bcolombe
 * Add ShowNewObject member
 *
 * Revision 1.54  2011/01/14 23:08:09  bcolombe
 * Add delegates for SET tooltip
 *
 * Revision 1.53  2011/01/07 22:02:03  bcolombe
 * Add UpOneLevel icon
 *
 * Revision 1.52  2011/01/07 17:58:37  bcolombe
 * Add LoadWebLayers that forces refresh
 *
 * Revision 1.51  2011/01/06 23:10:24  bcolombe
 * Remove use of v_AcceptItem in LIST
 *
 * Revision 1.50  2010/10/20 18:44:27  bcolombe
 * Changes to edit of new item in SET
 *
 * Revision 1.49  2010/10/14 23:01:48  bcolombe
 * Use tooltip delegate that provides canvas point
 *
 * Revision 1.48  2010/09/21 21:50:29  bcolombe
 * Add option to resolve autolink in CanSelectFileAsObject
 *
 * Revision 1.47  2010/09/17 20:58:53  bcolombe
 * Added ComputeExtentsForItem
 *
 * Revision 1.46  2010/09/15 23:17:11  bcolombe
 * Added SaveAs
 *
 * Revision 1.45  2010/09/07 22:28:38  bcolombe
 * Make CanSelectFileAsObject protected
 *
 * Revision 1.44  2010/09/03 20:56:32  bcolombe
 * Changes to switch navigate mode
 *
 * Revision 1.43  2010/09/02 22:59:34  bcolombe
 * LoadWebLayers takes item param
 *
 * Revision 1.42  2010/08/31 17:39:39  bcolombe
 * Added goto button
 *
 * Revision 1.41  2010/08/18 22:39:13  bcolombe
 * Add LoadWebLayers
 *
 * Revision 1.40  2010/08/16 18:51:56  bcolombe
 * Additional web layers changes
 *
 * Revision 1.39  2010/08/11 21:43:28  bcolombe
 * WebLayers dialog
 *
 * Revision 1.38  2010/06/28 19:44:04  mju
 * Add redrawBrowseList.
 *
 * Revision 1.37  2010/06/28 19:20:02  mju
 * Add hasUsedObjItem and virtual.
 *
 * Revision 1.36  2010/05/18 21:10:39  bcolombe
 * Make InitBrowseLocation virtual, add to SINGLE and MULTI dlgs
 *
 * Revision 1.35  2010/03/30 17:59:16  mju
 * Add skipItem.
 *
 * Revision 1.34  2010/03/26 14:37:12  mju
 * Remove old layout_page/book members.
 *
 * Revision 1.33  2010/03/24 21:39:32  mju
 * Add form_tabbed member.
 *
 * Revision 1.32  2010/02/17 22:42:25  mju
 * Add extvalue enum and related methods.
 *
 * Revision 1.31  2010/02/11 21:12:24  mju
 * Include objitemChild in dlg_getobject_child ctor.
 *
 * Revision 1.30  2009/12/21 14:56:32  mju
 * Change "new folder" icon to pushbutton and use simple label for "space free".
 *
 * Revision 1.29  2009/11/24 16:04:53  mju
 * Add members for moving new folder button to different toolbar.
 * Add setLabelNewFileName.
 *
 * Revision 1.28  2009/11/23 23:22:26  mju
 * Add v_enableNewFolder.
 *
 * Revision 1.27  2009/11/23 15:34:10  mju
 * Make onEditNewItemActivate protected.
 *
 * Revision 1.26  2009/11/20 22:17:55  mju
 * In "set" allow to reset item in used list via RMB menu.
 *
 * Revision 1.25  2009/11/13 15:37:07  mju
 * Add v_onClose overrides for list/set.
 *
 * Revision 1.24  2009/11/12 20:37:27  mju
 * Remove upOneLevel stuff as now does via navigatebar.
 *
 * Revision 1.23  2009/11/12 17:58:34  mju
 * Specify pane in createBrowseTypeControls.
 * Add control of used-list viewing mode.
 *
 * Revision 1.22  2009/11/12 17:22:03  mju
 * Use iconids directly instead of image list.
 *
 * Revision 1.21  2009/11/11 21:17:03  mju
 * Add bottomextra_None.
 *
 * Revision 1.20  2009/11/11 15:51:01  mju
 * Add toolbar button to navigate up one level.
 *
 * Revision 1.19  2009/11/11 15:13:50  mju
 * Give user choice for folders when add/AddAll.
 *
 * Revision 1.18  2009/11/09 18:48:26  mju
 * Add getShowErrorsRVC.
 *
 * Revision 1.17  2009/11/09 17:35:46  mju
 * Add members for user entering extension.
 *
 * Revision 1.16  2009/11/09 15:18:08  mju
 * Use std::set for extnSetAutoLinkRVC.
 *
 * Revision 1.15  2009/11/06 22:03:49  mju
 * Use const with filter so can use implicit ctors.
 *
 * Revision 1.14  2009/11/06 21:27:42  mju
 * Add canSelectFileAsObject.
 *
 * Revision 1.13  2009/11/05 21:50:07  mju
 * Always allow show all files, include selectable button if more than one type allowed.
 * Show extension with file type to save.
 *
 * Revision 1.12  2009/11/05 16:13:39  mju
 * Add dlg_single ctor using filepath.
 *
 * Revision 1.11  2009/11/04 22:57:43  mju
 * Make filetype separate class.
 *
 * Revision 1.10  2009/11/02 21:13:54  mju
 * Add members for text encoding.
 *
 * Revision 1.9  2009/11/02 20:20:16  mju
 * Add members for setting output file/object type.
 *
 * Revision 1.7  2009/10/29 21:49:01  mju
 * Stuff for new file types.
 *
 * Revision 1.6  2009/10/29 16:58:46  mju
 * Move formUsedList to list/set classes as not enough in common.
 *
 * Revision 1.5  2009/10/28 21:28:50  mju
 * Add setShowErrorRVC.
 *
 * Revision 1.4  2009/10/27 20:43:49  mju
 * Add setShowMaxPropDetails.
 *
 * Revision 1.1  2009/10/22 21:50:25  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_DLGGETOBJECTDLG_H
#define	INC_MGUI_DLGGETOBJECTDLG_H

#ifndef	INC_MGUI_DLGGETOBJECT_H
	#include <mgui/dlggetobject.h>
#endif

#ifndef	INC_MGUI_DLGSHELL_H
	#include <mgui/dlgshell.h>
#endif

#ifndef	INC_MGUI_EDIT_H
	#include <mgui/edit.h>
#endif

#ifndef	INC_MGUI_FORMLIST_H
	#include <mgui/formlist.h>
#endif

#ifndef	INC_MGUI_TOOLBAR_H
	#include <mgui/toolbar.h>
#endif

#ifndef	INC_MGUI_COMBOBOX_H
	#include <mgui/combobox.h>
#endif

#ifndef	INC_MGUI_STATUSBAR_H
	#include <mgui/statusbar.h>
#endif

#ifndef	INC_MGUI_SASH_H
	#include <mgui/sash.h>
#endif

#ifndef	INC_MGUI_FORMRADIOBUTTONS_H
	#include <mgui/formradiobuttons.h>
#endif

#ifndef	INC_MGUI_FORMTABBED_H
	#include <mgui/formtabbed.h>
#endif

#ifndef	INC_RVC_OBJITEMLIST_H
	#include <rvc/objitemlist.h>
#endif

#ifndef	INC_MI32_MISTRINGLIST_H
	#include <mi32/mistringlist.h>
#endif

#ifndef	INC_MI32_BITSET_H
	#include <mi32/bitset.h>
#endif

#ifndef INC_MI32_XML_H
	#include <mi32/xml.h>
#endif

#ifndef	INC_SOA_UISERVERLIST_H
	#include <soa/uiserverlist.h>
#endif

#ifndef INC_SPATMOD_IMAGETILESET_H
	#include <spatmod/imagetileset.h>
#endif

#include <set>

#ifndef GENERATING_DOXYGEN_OUTPUT
class MINETRES;
class MISTATUSCONTEXT;
namespace SOA {
	namespace WMS {
		class UI_FORM_CONNECTION;
		}
	namespace ARCIMS {
		class UI_FORM_CONNECTION;
		}
	}
namespace GRE{
	class GROUP;
	namespace UI{
		class FORM_VIEW;
		}
	}
namespace MGUI {
	class FORM_GEOCATALOG;
	class DLG_FUSIONTABLES;
	class DLG_TEXTVIEW;
	class FORM_GOOGLESEARCH;
	}
#endif


namespace MGUI {

//=====================================================================================================================

//! Base for all object selection dialogs.
class DLG_GETOBJECT : public MGUI::DLGSHELL {
	public:

		//! Destructor.
		virtual ~DLG_GETOBJECT ();

		//! Get selected text encoding.
		CHAR_ENCODING GetTextEncoding (
			) const { return (m_TextEncoding); }

		//! Set delegate to create custom controls at bottom of dialog, just above buttons.
		void SetDelegateCreateCustomControlsBottom (
			DELEGATE_CREATECUSTOMCONTROLS delegate
			) { m_DelegateCreateCustomControlsBottom = delegate; }

		//! Set delegate to create custom controls at top of dialog, just below prompt if any.
		void SetDelegateCreateCustomControlsTop (
			DELEGATE_CREATECUSTOMCONTROLS delegate
			) { m_DelegateCreateCustomControlsTop = delegate; }

		//! Set label for "New File Name" control.
		//! Used to override.
		void SetLabelNewFileName (
			MISTRING label
			) { m_LabelNewFileName = label; }

		//! Set message to display if overwriting existing file/object
		void SetOverwriteMessage (
			const MISTRING& message
			) { m_OverwriteMessage = message; }

		//! Set text encoding (before dialog created).
		void SetTextEncoding (
			CHAR_ENCODING encoding
			) { m_TextEncoding = encoding; }

		class ITEM;
		typedef MILIST<ITEM> ITEMLIST;

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		enum BOTTOMEXTRA {
			BOTTOMEXTRA_None,
			BOTTOMEXTRA_BrowseType,
			BOTTOMEXTRA_NewItemType,
			BOTTOMEXTRA_TextEncoding,
			BOTTOMEXTRA_Custom,
			};

		//! Constructor.
		DLG_GETOBJECT (
			const MISTRING& title,
			const DLG_GETOBJECT_FILTER& filter,
			DLG_GETOBJECT_STYLE dlgstyle,
			const char *IniGroupSettings,
			const char *IniGroupDftPath,
			const MISTRING *pPromptStr = 0,
			bool ShowNewObject = true
			);

		int AcceptItem (
			ITEM& item,
			bool showmsg
			);

		bool AcceptNewItem ();

		bool AcceptNewItem (const MISTRING& name, const MISTRING& description);

		int AcceptObjItem (
			const RVC::OBJITEM& objitem,
			bool isfile,
			bool showmsg
			);

		void AddToolbarItemNavigateMode (CTRL_TOOLBAR& toolbar);
		void AddToolbarItemRefresh (CTRL_TOOLBAR& toolbar);
		void AddToolbarItemUpOneLevel (CTRL_TOOLBAR& toolbar);
		void AddToolbarItemViewMode (CTRL_TOOLBAR& toolbar);

		void BuildBrowseList (bool ResetNewItem);

		ERRVALUE BuildFileList (
			const FILEPATH& FolderPath,
			ITEMLIST& ItemList
			) const;

		bool CanAddToFavorites() const;
		bool CanDelete () const;
		bool CanEditACL () const;
		bool CanRename () const;
		bool CanSaveAs() const;
		bool CanSelectItemAsObject (
			const ITEM& item,
			RVC::OBJITEM &ObjItemRet,
			bool ResolveAutoLink = true
			) const;
		bool CanShowMetadata () const;
		bool CanShowProperties () const;
		bool CanViewInGoogleMaps() const;

		void CheckEnableOK ();

		//! Clear statusbar text.
		void ClearMessage (
			) { m_StatusBar.ClearText(); }

		//! Create controls at bottom of dialog.
		void CreateBottomControls (
			BOTTOMEXTRA BottomExtra,
			bool AllowNew
			);

		//! Create controls for browsing and selecting.
		ERRVALUE CreateBrowseControls (
			LAYOUT_PANE_BASE& PaneParent
			);

		//! Create list for browsing and selecting.
		void CreateBrowseList (
			LAYOUT_PANE_BASE& PaneParent
			);

		//! Create controls for choosing item types to show/select.
		void CreateBrowseTypeControls (
			LAYOUT_PANE_BASE& PaneParent
			);

		//! Create controls for reporting available space.
		void CreateControlsSpaceAvailable (
			LAYOUT_PANE_BASE& PaneParent,
			bool ShowTemporary = true
			);

		//! Create custom controls at bottom of dialog, just above buttons.
		void CreateCustomControlsBottom ();

		//! Create custom controls at top of dialog, just below prompt if any.
		void CreateCustomControlsTop ();

		//! Create label to show prompt.
		void CreateLabelPrompt (
			);

		//! Create controls for navigation.
		void CreateNavigateControls (
			LAYOUT_PANE_BASE& PaneParent,
			bool IncludeToolbar = true
			);

		//! Create button for new file
		void CreateNewItemEditButton (
			);

		//! Create controls for new file/object.
		void CreateNewItemEditControls (
			);

		//! Create controls for choosing new item type.
		void CreateNewItemTypeControls (
			bool UseButtonPane = false
			);

		// Create statusbar control.
		void CreateStatusBar ();

		//! Create controls for text encoding.
		void CreateTextEncodingControls (
			bool UseButtonPane = false
			);

		void DeselectAllItems ();

		void DoCmdAddToFavorites ();
		void DoCmdDeleteSelectedItems ();
		ERRVALUE DoCmdEditACL ();
		void DoCmdNewFolder ();
		void DoCmdRename ();
		void DoCmdSaveAs ();
		void DoCmdShowMetadata ();
		void DoCmdShowProperties ();
		void DoCmdViewInGoogleMaps ();

		ITEM& GetBrowseListItem (int itemnum) const
			{ return (*reinterpret_cast<ITEM*>(m_FormBrowseList.GetItemID(itemnum))); }

		ITEM * GetBrowseListItemSelected () const;
		const RVC::OBJITEM* GetBrowseListObjItemSelected () const;

		//! Get dialog style.
		DLG_GETOBJECT_STYLE GetDlgStyle (
			) const { return (m_dlgstyle); }

		const FORM_LIST& GetFormBrowseList (
			) const { return (m_FormBrowseList); }

		FORM_GEOCATALOG& GetFormCatalog (
			) const { return (*m_pFormCatalog); }

		SOA::ARCIMS::UI_FORM_CONNECTION& GetFormArcIMS (
			) const { return (*m_pFormArcIMS); }

		SOA::WMS::UI_FORM_CONNECTION& GetFormWMS (
			) const { return (*m_pFormWMS); }

		RVC::OBJTYPE GetObjTypeShow (
			) const { return (m_ObjTypeShow); }

		INT64 GetSpaceFreeInFile (
			) const { return (m_SpaceFreeInFile); }

		//! Get RVC::OBJECT for current location.
		const RVC::OBJECT& GetObjParent (
			) const { return (*m_pObjParent); }

		const DLG_GETOBJECT_FILTER& GetFilter (
			) const { return (*m_pFilter); }

		const char * GetIniGroupSettings (
			) const { return (m_IniGroupSettings); }

		const ITEM& GetItemBrowseLocation (
			) const { return (*m_pItemBrowseLocation); }

		const ITEMLIST& GetItemListBrowse (
			) const { return (*m_pItemListBrowse); }

		void GetItemToolTip (CTRL_TOOLTIP& ToolTip, ITEM& item, CTRL* MenuParent) const;

		const MISTRING& GetNewFileTypeExtn (
			) const;

		RVC::OBJTYPE GetObjTypeNew (
			) const { return (m_ObjTypeNew); }

		//! Get type of selected item, if none or multiple may return TYPE_Invalid.
		int GetSelectedItemType () const;

		bool GetShowErrorsRVC (
			) const { return (m_ShowErrorsRVC); }

		//! Get whether to use values from NewItem controls.
		bool GetUseNewItem (
			) const;

		bool HasCustomControlsBottom (
			) const { return (!!m_DelegateCreateCustomControlsBottom); }

		bool HasBrowseItemAvailable () const;
		bool HasBrowseItemSelected () const;

		void InitFilePatternLists ();

		bool IsBrowseListShown () const;
		bool IsCatalogResultShown () const;

		//! Determine if name is unique for new child of type m_ObjTypeTestUnique.
		int IsNewChildShortNameUnique (
			const RVC::OBJECTNAME& name
			) const;

		bool IsNewItemTypeRVC (
			) const { return (m_ObjTypeNew != RVC::OBJTYPE_LAST); }

		//! Determine if html is valid geomashup
		bool IsValidGeomashup (
			const FILEPATH& path
			) const;

		//! Determine if item is in "used" list.
		bool HasUsedObjItem (
			const RVC::OBJITEM& objitem
			) const { return (v_HasUsedObjItem(objitem)); }

		// Notify that existing item in browse list has changed.
		void NotifyChangeBrowseListItem (
			int itemnum
			);

		void OnEditNewItemActivate ();

		int PromptRecurseLevels (MISTRING& pattern) const;

		void RedrawBrowseList ()
			{ m_FormBrowseList.Redraw(); }

		void ResetEditNewItem ();

		void SaveBrowseLocation ();

		//! Set current location using FILEPATH.
		void SetBrowseLocation (
			const FILEPATH& FilePath,
			bool UpdateNavigateBar = true
			);

		//! Set current location from OBJITEM.
		void SetBrowseLocation (
			const RVC::OBJITEM& ObjItemLocation,
			bool UpdateNavigateBar = true
			);

		//! Set current location from ITEM.
		void SetBrowseLocation (
			const ITEM& Item,
			bool UpdateNavigateBar = true
			);

		void SetMessage (
			const MISTRING& message,
			bool UpdateNow = false
			);

		void SetMessageError (
			ERRVALUE err
			);

		void SetObjTypeTestUnique (
			RVC::OBJTYPE ObjType
			) { m_ObjTypeTestUnique = ObjType; }

		void SetShowErrorsRVC (
			bool ShowErrorsRVC = true
			) { m_ShowErrorsRVC = ShowErrorsRVC; }

		void SetShowMaxPropDetails (
			bool ShowMaxPropDetails = true
			) { m_ShowMaxPropDetails = ShowMaxPropDetails; }

		void UpdateSpaceFree ();

		virtual void v_InitBrowseLocation ();
		virtual void v_OnClose ();
		virtual void v_OnOK ();

		#endif // GENERATING_DOXYGEN_OUTPUT
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLGSHELL BASECLASS;
		typedef DLG_GETOBJECT THISCLASS;
		class NAVIGATEBAR;

		enum EXTVALUES {
			EXTVALUE_None =				0,
			EXTVALUE_Description = 		0x0001,
			EXTVALUE_TimeCreated =		0x0002,
			EXTVALUE_TimeModified =		0x0004,
			EXTVALUE_Dimensions =		0x0008,
			EXTVALUE_CRS =					0x0010,
			EXTVALUE_ResolutionInCRS =	0x0020,
			EXTVALUE_ExtentInCRS =		0x0040,
			EXTVALUE_SubType =			0x0080,
			EXTVALUE_SizeInBytes =		0x0100,
			};

		MISTRING m_promptstr;
		MISTRING m_LabelNewFileName;
		MISTRING m_OverwriteMessage;
		const DLG_GETOBJECT_FILTER *m_pFilter;
		DLG_GETOBJECT_STYLE m_dlgstyle;
		DELEGATE_CREATECUSTOMCONTROLS m_DelegateCreateCustomControlsBottom;
		DELEGATE_CREATECUSTOMCONTROLS m_DelegateCreateCustomControlsTop;
		CTRL_LABEL m_LabelPrompt;
		NAVIGATEBAR *m_pNavigateBar;
		CTRL_TOOLBAR m_ToolbarNavigate;
		CTRL_TOOLBAR m_ToolbarPath;
		FORM_TABBED m_TabFormBrowse;
		FORM_TABBED m_TabFormCatalog;
		FORM_TABBED m_TabFormSearchWeb;
		FORM_LIST m_FormBrowseList;
		FORM_GEOCATALOG *m_pFormCatalog;
		DLG_FUSIONTABLES *m_pDlgFusionTable;
		SOA::ARCIMS::UI_FORM_CONNECTION *m_pFormArcIMS;
		SOA::WMS::UI_FORM_CONNECTION *m_pFormWMS;
		FORM_GOOGLESEARCH *m_pFormGoogleSearch;
		CTRL_PUSHBUTTON m_BtnNewFolder;
		CTRL_LABEL m_LabelSpaceFree;
		CTRL_LABEL m_LabelTempSpace;
		FORM_EDIT_STRING m_EditNewItemName;
		FORM_EDIT_STRING m_EditNewItemDesc;
		CTRL_PUSHBUTTON m_BtnCreateFile;
		FORM_COMBOBOX m_CbxObjTypeShow;
		CTRL_LABEL m_LblFileType;
		FORM_RADIOBUTTONS m_RbnFileType;
		CTRL_COMBOBOX m_CbxFileTypeShow;
		CTRL_EDIT_STRING m_EditFileTypeShow;
		FORM_COMBOBOX m_CbxNewItemType;
		FORM_COMBOBOX m_CbxTextEncoding;
		CTRL_PUSHBUTTON m_BtnSkip;
		CTRL_STATUSBAR m_StatusBar;
		int m_PageBrowse;
		int m_PageCatalog;
		int m_PageCatalogSearch;
		int m_PageCatalogResults;
		int m_PageCatalogList;
		int m_PageWebSearch;
		int m_PageArcIMS;
		int m_PageWMS;
		int m_PageGoogleSearch;
		int m_BrowseFieldName;
		int m_BrowseFieldDateModified;
		int m_BrowseFieldDateCreated;
		int m_BrowseFieldSize;
		int m_BrowseFieldType;
		int m_BrowseFieldDescription;
		int m_BrowseFieldDimensions;
		int m_BrowseFieldSuccessCount;
		int m_BrowseFieldFailureCount;
		int m_BrowseFieldReliability;
		FORM_LIST::VIEWMODE m_BrowseListViewMode;
		int m_BrowseListActiveItemNum;
		int m_BrowseListLastPressItemNum;
		ITEM *m_pItemBrowseLocation;
		ITEMLIST *m_pItemListBrowse;
		const FILETYPE *m_pFileTypeNew;
		RVC::OBJTYPE m_ObjTypeShow;			// Which object types to show (OBJTYPE_LAST means 'Selectable')
		RVC::OBJTYPE m_ObjTypeNew;				// Type of new object
		RVC::OBJTYPE m_ObjTypeTestUnique;	// Type of new object for uniqueness test
		RVC::OBJECT *m_pObjParent;				// Current RVC file
		const char *m_IniGroupSettings;
		char *m_IniGroupDftPath;
		char m_IniNameDftPath[60];
		bool m_AllowShowObjTypesRVC;
		bool m_AllowShowLinkTypesRVC;
		bool m_ShowErrorsRVC;
		bool m_ShowMaxPropDetails;
		CHAR_ENCODING m_TextEncoding;
		MISTRINGLIST m_FilePatternListMenu;
		MISTRINGLIST m_FilePatternListUsed;
		mutable std::set<MISTRING> m_ExtnSetAutoLinkRVC;
		MISTRING m_LastUsedFilePattern;
		MISTRINGLIST m_LastUsedFilePatternList;
		bool m_UseNewItem;
		bool m_ShowNewObject;
		DLG_TEXTVIEW *m_pDlgProperties;
		INT64 m_SpaceFreeInFile;
		SPATMOD::IMAGE::TILESET::LIST m_tilesetlist;
		ERRVALUE m_ErrDownloading;

		void AddRbnFilePattern (MISTRINGLIST::CONST_ITERATOR& itFilePattern, MISTRINGLIST& RbnFilePatternList);

		ERRVALUE BuildBrowseListComputer ();
		ERRVALUE BuildBrowseListFolder ();
		ERRVALUE BuildBrowseListGeomashup ();
		ERRVALUE BuildBrowseListNetwork ();
		ERRVALUE BuildBrowseListWebLayers ();
		ERRVALUE BuildBrowseListRVC ();

		EXTVALUES CalcExtValuesNeed (int fieldnum) const;

		void ComputeExtentsForItem (ITEM& item) const;

		ERRVALUE DeleteFolder (const FILEPATH& folderpath, bool DoReadOnly);

		bool GetDftShowOnlyRVCFiles () const;

		bool GetNewObjItem (
			RVC::OBJITEM& objitem
			) { return (v_GetNewObjItem(objitem)); }

		bool IsNodeBrowsable (
			XMLNODE* rootnode
			);

		ERRVALUE DownloadMashup(XMLNODE* rootnode);
		ERRVALUE DownloadServerList(XMLNODE* rootnode);

		void LoadLastUsedFileTypes ();
		ERRVALUE LoadWebLayers (ITEM& item);
		ERRVALUE LoadWebLayers (ITEM& item, bool bForceRefresh);

		void OnBrowseListChangeSelection (INT32 itemnum);
		bool OnBrowseListDrawItem (FORM_LIST::CUSTOMDRAW& customdraw) const;
//		bool OnBrowseListEditBegin (int itemnum, int fieldnum, MISTRING& value);
//		bool OnBrowseListEditEnd (int itemnum, int fieldnum, MISTRING& value);
		int OnBrowseListFieldCompare (int itemnum1, int itemnum2, int fieldnum) const;
		void OnBrowseListGetDispInfo (int itemnum, int fieldnum, FORM_LIST::DISPINFO& dispinfo) const;
		bool OnBrowseListDoubleClick (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);
		bool OnBrowseListPointerButtonDown (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);
		bool OnBrowseListPointerButtonUp (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);
		bool OnBrowseListToolTip (int itemnum, int fieldnum, CTRL_TOOLTIP &ToolTip, const POINT& Point) const;
		void OnCatalogClickItem (const RVC::OBJITEM& objitem);
		void OnCatalogDoubleClickItem (const RVC::OBJITEM& objitem);
		void OnCatalogCmdSearch ();
		void OnCbxFileTypeShow ();
		void OnCbxNewItemType ();
		void OnCbxObjTypeShow ();
		void OnCbxTextEncoding ();
		void OnCmdNavigateMode (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdNewFolder (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdRefresh (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdSkip ();
		void OnCmdUpOneLevel (MGUI::ID cmdid, bool ToggleOn);
		void OnEditFileTypeShowApply ();
		void OnEditNewItemUserEditName ();
		void OnGetFusionTable (XMLNODE* node);
		void OnMenuBrowseListViewMode (MGUI::ID id, bool ToggleOn);
		void OnNavigateChangeLocation ();
		void OnNavigateEditEnd ();
		bool OnNavigateSelectItem(const ITEM& item);
		bool OnNavigateTestLocation (const RVC::OBJITEM& location) const;
		void OnPasteAndGo (MGUI::ID cmdid, bool ToggleOn);
		void OnPageActivate ();
		void OnRbnFileType ();
		void OnArcIMSDoubleClickItem (SOA::SERVERLIST::ITEM& item);
		void OnWMSDoubleClickItem (SOA::SERVERLIST::ITEM& item);
		void OnKMLDoubleClickItem (const MISTRING& name, const MISTRING& url);

		void SaveLastUsedFileTypes ();
		void SetFilePattern (const MISTRING *pFilePattern, bool OnlySelectable = true);
		int TestShortNameUnique (const RVC::OBJECTNAME& name);
		void UpdateDlgProperties ();

		// Return TRUE if accepted, FALSE if not, < 0 if error
		virtual int v_AcceptObjItem (const RVC::OBJITEM& objitem, bool isfile) = 0;
		virtual int v_AcceptItem (ITEM& item, bool showmsg);
		virtual void v_AddedItemsEnd();
		virtual int v_AddToFavorites (ITEM& item);
		virtual bool v_CanAddToFavorites () const;
		virtual bool v_CheckEnableOK ();
		virtual void v_EnableNewFolder (bool);
		virtual bool v_GetNewObjItem (RVC::OBJITEM& objitem);
		virtual bool v_HasUsedObjItem (const RVC::OBJITEM& objitem) const;
		virtual void v_OnDeleteDone ();
		virtual void v_OnUpdateUI ();

		virtual void v_OnOpen ();

		#endif
	};


//=====================================================================================================================

//! Single-object selection dialog.
class DLG_GETOBJECT_SINGLE : public DLG_GETOBJECT {
	public:

		//! Constructor with default OBJITEM.
		DLG_GETOBJECT_SINGLE (
			const MISTRING& title,
			const DLG_GETOBJECT_FILTER& filter,
			const RVC::OBJITEM& ObjItemSelected,
			DLG_GETOBJECT_STYLE dlgstyle,
			const char *IniGroupDftPath = 0,
			const MISTRING *pPromptStr = 0
			);

		//! Constructor with default FILEPATH.
		DLG_GETOBJECT_SINGLE (
			const MISTRING& title,
			const DLG_GETOBJECT_FILTER& filter,
			const FILEPATH& FilePathSelected,
			DLG_GETOBJECT_STYLE dlgstyle,
			const char *IniGroupDftPath = 0,
			const MISTRING *pPromptStr = 0
			);

		//! Destructor.
		virtual ~DLG_GETOBJECT_SINGLE ();

		//! Get last selected item.
		const RVC::OBJITEM& GetSelectedItem (
			) const { return (m_ObjItemSelected); }

	protected:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Internal constructor used by DLG_GETOBJECT_CHILD
		DLG_GETOBJECT_SINGLE (
			const MISTRING& title,
			const RVC::OBJITEM& ObjItemChild,
			const DLG_GETOBJECT_FILTER& filter,
			DLG_GETOBJECT_STYLE dlgstyle,
			const char *IniGroupSettings,
			const MISTRING *pPromptStr = 0
			);

		virtual void v_InitBrowseLocation ();
		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_GETOBJECT BASECLASS;
		typedef DLG_GETOBJECT_SINGLE THISCLASS;

		RVC::OBJITEM m_ObjItemSelected;

		void AcceptSelected ();

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOK ();
		virtual int v_AcceptObjItem (const RVC::OBJITEM& objitem, bool isfile);
		virtual bool v_CheckEnableOK ();
		virtual bool v_GetNewObjItem (RVC::OBJITEM& objitem);
		#endif
	};

//=====================================================================================================================

//! Child object selection dialog.
class DLG_GETOBJECT_CHILD : public DLG_GETOBJECT_SINGLE {
	public:

		//! Constructor.
		DLG_GETOBJECT_CHILD (
			const MISTRING& title,
			const RVC::OBJITEM& ObjItemParent,
			const DLG_GETOBJECT_FILTER& filter,
			const RVC::OBJITEM& ObjItemChild,
			DLG_GETOBJECT_STYLE dlgstyle,
			const MISTRING *pPromptStr = 0
			);

		//! Destructor.
		virtual ~DLG_GETOBJECT_CHILD ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_GETOBJECT_SINGLE BASECLASS;
		#endif
	};

//=====================================================================================================================

//! Base for multi-object selection dialogs.
class DLG_GETOBJECT_MULTI : public DLG_GETOBJECT {
	public:

		//! Destructor.
		virtual ~DLG_GETOBJECT_MULTI ();

		//! list of selected items.
		void GetSelectedItems (
			RVC::OBJITEMLIST& ObjItemList
			) const;

		//! list of selected items.
		void GetSelectedItems (
			ITEMLIST& ItemList
			) const;

		//! Set selected items.
		ERRVALUE SetSelectedItems (
			const RVC::OBJITEMLIST& ObjItemList
			);

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		ITEMLIST m_ItemListUsed;
		ITEMLIST m_ItemListFavorites;
		ITEMLIST m_ItemListRecent;

		//! Constructor.
		DLG_GETOBJECT_MULTI (
			const MISTRING& title,
			const DLG_GETOBJECT_FILTER& filter,
			DLG_GETOBJECT_STYLE dlgstyle,
			const char *IniGroupSettings,
			const char *IniGroupDftPath = 0,
			const MISTRING *pPromptStr = 0,
			bool ShowNewObject = true
			);

		void CreateSash (
			LAYOUT_PANE_BASE& PaneParent
			);

		virtual void v_InitBrowseLocation ();
		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_GETOBJECT BASECLASS;
		typedef DLG_GETOBJECT_MULTI THISCLASS;

		CTRL_SASH m_Sash;

		virtual bool v_HasUsedObjItem (const RVC::OBJITEM& objitem) const;
		virtual ERRVALUE v_SetSelectedItemsBegin (const RVC::OBJITEMLIST& ObjItemList);
		virtual void v_SetSelectedItemsEnd () = 0;
		#endif
	};

//=====================================================================================================================

//! Select variable list of objects.
class DLG_GETOBJECT_LIST : public DLG_GETOBJECT_MULTI {
	public:

		//! Constructor.
		DLG_GETOBJECT_LIST (
			const MISTRING& title,
			const DLG_GETOBJECT_FILTER& filter,
			DLG_GETOBJECT_STYLE dlgstyle,
			int minobjs = 1,							//!< Minimum number of objects required
			int maxobjs = 0,							//!< Maximum allowed to be selected, 0 if no limit
			const char *IniGroupDftPath = 0,
			const MISTRING *pPromptStr = 0
			);

		//! Destructor.
		virtual ~DLG_GETOBJECT_LIST ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_GETOBJECT_MULTI BASECLASS;
		typedef DLG_GETOBJECT_LIST THISCLASS;

		enum CMDID {
			CMDID_Add =		1,
			CMDID_AddAll,
			CMDID_AddAllSub,
			CMDID_Remove,
			CMDID_RemoveAll,
			CMDID_AddToFavorites,
			CMDID_Rename
			};

		int m_minobjs;
		int m_maxobjs;
		int m_UsedFieldName;
		int m_UsedFieldLocation;
		int m_FavoritesFieldName;
		int m_FavoritesFieldLocation;
		int m_RecentFieldName;
		int m_RecentFieldLocation;
		int m_PageSelected;
		int m_PageFavorites;
		int m_PageRecent;
		int m_PagePreview;
		GRE::GROUP* m_PreviewGroup;
		GRE::UI::FORM_VIEW* m_PreviewView;
		CTRL_TOOLBAR m_ToolbarUsedList;
		CTRL_TOOLBAR m_ToolbarFavoritesList;
		CTRL_TOOLBAR m_ToolbarRecentList;
		FORM_LIST m_FormUsedList;
		FORM_LIST m_FormFavoritesList;
		FORM_LIST m_FormRecentList;
		FORM_TABBED m_TabForm;
		FORM_LIST::VIEWMODE m_UsedListViewMode;
		FORM_LIST::VIEWMODE m_FavoritesListViewMode;
		FORM_LIST::VIEWMODE m_RecentListViewMode;

		ERRVALUE AddAll (
			);

		ERRVALUE AddAllFavorites (
			);

		ERRVALUE AddAllRecent (
			);

		ERRVALUE AddAll (
			const RVC::OBJECT& ObjParent,
			bool recursefolders,
			const MISTRING& pattern
			);

		ERRVALUE AddAll (
			const RVC::OBJECT& ObjParent,
			bool recursefolders
			);

		ERRVALUE AddAllChildObjects (
			const RVC::OBJECT& ObjParent,
			const MISTRING& pattern
			);

		ERRVALUE AddAllRecursive (
			const FILEPATH& FolderPath,
			MISTATUSCONTEXT& StatusContext,
			int RecurseLevels,
			const MISTRING& pattern
			);

		int AddSelected ();

		int AddSelectedFavorites ();

		int AddSelectedRecent ();

		void AppendToUsedList (const ITEM *pItem);
		void AppendToFavoritesList (const ITEM *pItem);
		void AppendToRecentList (const ITEM *pItem);
		void CreatePreview();
		void OnCmdUsedList (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdFavoritesList (MGUI::ID cmdid, bool ToggleOn);
		void OnCmdRecentList (MGUI::ID cmdid, bool ToggleOn);
		void OnMenuUsedListViewMode (MGUI::ID id, bool ToggleOn);
		void OnMenuFavoritesListViewMode (MGUI::ID id, bool ToggleOn);
		void OnMenuRecentListViewMode (MGUI::ID id, bool ToggleOn);
		void OnPreviewActive ();
		void OnFavoritesActive ();
		void OnRecentActive ();
		void OnUsedListChangeSelection (INT32 itemnum);
		void OnFavoritesListChangeSelection (INT32 itemnum);
		void OnRecentListChangeSelection (INT32 itemnum);
		bool OnFavoritesListDoubleClick (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);
		bool OnRecentListDoubleClick (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);		
		bool OnUsedListPointerButtonDown (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);
		bool OnFavoritesListPointerButtonDown (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);
		bool OnRecentListPointerButtonDown (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);
		bool OnFavoritesListPointerButtonUp (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);
		bool OnRecentListPointerButtonUp (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);
		bool OnUsedListToolTip (int itemnum, int fieldnum, CTRL_TOOLTIP &ToolTip, const POINT& Point) const;
		bool OnFavoritesListToolTip (int itemnum, int fieldnum, CTRL_TOOLTIP &ToolTip, const POINT& Point) const;
		bool OnRecentListToolTip (int itemnum, int fieldnum, CTRL_TOOLTIP &ToolTip, const POINT& Point) const;
		void PopulateFavorites();
		void PopulateRecent();
		void SaveFavoriteItems();
		void SaveRecentItems();

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnClose ();
		virtual void v_OnOK ();
		virtual int v_AcceptObjItem (const RVC::OBJITEM& objitem, bool isfile);
		virtual int v_AddToFavorites (ITEM& item);
		virtual bool v_CanAddToFavorites () const;
		virtual bool v_CheckEnableOK ();
		virtual void v_OnDeleteDone ();
		virtual void v_OnUpdateUI ();
		virtual void v_SetSelectedItemsEnd ();
		virtual void v_AddedItemsEnd();
		#endif
	};

//=====================================================================================================================

//! Select set of objects.
class DLG_GETOBJECT_SET : public DLG_GETOBJECT_MULTI {
	public:

		//! Construct with specified set of items.
		DLG_GETOBJECT_SET (
			const MISTRING& title,
			const MISTRING& LabelHeading,
			const DLG_GETOBJECT_FILTER& filter,
			DLG_GETOBJECT_STYLE dlgstyle,
			const RVC::OBJITEMLIST& objitems,
			const char *IniGroupDftPath = 0,
			const MISTRING *pPromptStr = 0
			);

		//! Construct with specified number of items.
		DLG_GETOBJECT_SET (
			const MISTRING& title,
			const MISTRING& LabelHeading,
			const DLG_GETOBJECT_FILTER& filter,
			DLG_GETOBJECT_STYLE dlgstyle,
			int numitems,								//!< Number of items in set
			TEXTID LabelPrefix = TEXTID__None,	//!< Label prefix, will derive from allowed object type(s) if none
			const char *IniGroupDftPath = 0,
			const MISTRING *pPromptStr = 0
			);

		//! Destructor.
		virtual ~DLG_GETOBJECT_SET ();

		//! Set label for item in set.
		void SetLabel (
			int itemidx,
			const MISTRING& label
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLG_GETOBJECT_MULTI BASECLASS;
		typedef DLG_GETOBJECT_SET THISCLASS;

		MISTRING m_LabelHeading;
		MISTRING *m_Labels;
		BITSET m_UsedSet;
		CTRL_GROUPBOX m_GbxUsedList;
		FORM_LIST m_FormUsedList;
		int m_UsedFieldLabel;
		int m_UsedFieldName;
		int m_UsedFieldDescription;
		int m_UsedFieldLocation;
		CTRL_PUSHBUTTON m_BtnAutoName;
		RVC::OBJITEM m_PrevObjItem;

		ERRVALUE DoCmdAutoNameFiles ();
		ERRVALUE DoCmdAutoNameObjects ();
		bool OnEditBegin(int itemnum, int fieldnum, MISTRING& string);
		bool OnEditCancel(int itemnum, int fieldnum);
		bool OnEditEnd(int itemnum, int fieldnum, MISTRING& string);
		void OnCmdAutoName ();
		void OnUsedListChangeSelection (INT32);
		bool OnUsedListPointerButtonUp (int itemnum, int fieldnum, const MGUI::POINT& pt, POINTERBUTTON button, KEYSTATE keystate);
		void ResetUsedListItem (int itemnum);
		void SkipUsedListItem (int itemnum);
		void UpdateUsedListItem (int itemnum);

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnClose ();
		virtual void v_OnOK ();
		virtual int v_AcceptObjItem (const RVC::OBJITEM& objitem, bool isfile);
		virtual bool v_CheckEnableOK ();
		virtual bool v_GetNewObjItem (RVC::OBJITEM& objitem);
		virtual void v_OnDeleteDone ();
		virtual ERRVALUE v_SetSelectedItemsBegin (const RVC::OBJITEMLIST& ObjItemList);
		virtual void v_SetSelectedItemsEnd ();
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_DLGGETOBJECTDLG_H
