/**
 * \file <mgui/dlgmymaps.h>
 *
 * \if NODOC
 * $Id: dlgmymaps.h_v 1.4 2011/11/21 21:27:28 mju Exp $
 *
 * $Log: dlgmymaps.h_v $
 * Revision 1.4  2011/11/21 21:27:28  mju
 * Remove redundant hdr.
 *
 * Revision 1.3  2011/06/27 14:15:28  mju
 * Fix doc warning and inclusion guards.
 *
 * Revision 1.2  2010/08/24 22:59:47  bcolombe
 * Add toggle for remember password
 *
 * Revision 1.1  2010/08/11 21:44:43  bcolombe
 * Initial revision
 * \endif
****************************************************************************/

#ifndef	INC_MGUI_DLGMYMAPS_H
#define	INC_MGUI_DLGMYMAPS_H

#ifndef	INC_MGUI_DLGSHELL_H
	#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_EDIT_H
	#include <mgui/edit.h>
#endif

#ifndef INC_MI32_XML_H
	#include <mi32/xml.h>
#endif

namespace MGUI {

class DLG_MYMAPS : public MGUI::DLGSHELL {
	public:
		typedef fastdelegate::FastDelegate<void(XMLNODE*)> DELEGATE_ONGETMYMAPS;

		DLG_MYMAPS (XMLNODE* node);

		~DLG_MYMAPS();

		//! Generate My Maps list
		void GetMyMaps(bool notify = false);

		//! Launch google maps to view specified mymaps url
		void RunGoogleMaps(MISTRING url);

		//! Set delegate to call when mymaps have been successfully retrieved
		//! Passes node that was created containing mymaps items
		void SetDelegateOnGetMyMaps (
			DELEGATE_ONGETMYMAPS delegate
			) { m_DelegateOnGetMyMaps = delegate; }

		//! Set node to create mymaps items under
		void SetRootNode(XMLNODE* node) { m_rootnode = node; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		void EnableGetMyMapsBtn();
		void DisableGetMyMapsBtn();

		XMLNODE* m_rootnode;
		DELEGATE_ONGETMYMAPS m_DelegateOnGetMyMaps;

		MGUI::CTRL_EDIT_STRING m_UserName;
		MGUI::CTRL_EDIT_STRING m_Password;
		MGUI::CTRL_TOGGLEBUTTON m_Remember;
		MGUI::CTRL_TEXT m_Content;

		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnOK ();
		#endif
	};

} //end MGUI namespace

#endif //INC_MGUI_DLGMYMAPS_H
