/** 
 * \file <mgui/formlinktotileset.h>
 * \brief Definitions for MGUI::DLG_LINKTOTILESET class.
 *
 * \if NODOC
 * $Id: formlinktotileset.h_v 1.4 2011/06/27 14:11:09 mju Exp $
 *
 * $Log: formlinktotileset.h_v $
 * Revision 1.4  2011/06/27 14:11:09  mju
 * *** empty log message ***
 *
 * Revision 1.3  2010/09/20 16:18:53  bcolombe
 * Add make tile map option
 *
 * Revision 1.2  2010/03/19 17:16:06  bcolombe
 * Dont use dlgshell for level or type selection
 *
 * Revision 1.1  2010/02/01 21:45:12  bcolombe
 * Initial revision
 *
 *
 * \endif
**/

#ifndef	INC_MGUI_DLG_LINKTOTILESET_H
#define	INC_MGUI_DLG_LINKTOTILESET_H

#ifndef INC_MGUI_DLGSHELL_H
#include <mgui/dlgshell.h>
#endif

#ifndef INC_MGUI_TOOLBAR_H
#include <mgui/toolbar.h>
#endif

#ifndef INC_MGUI_LISTBOX_H
#include <mgui/listbox.h>
#endif

#ifndef INC_MGUI_FORMLIST_H
#include <mgui/formlist.h>
#endif

#ifndef INC_MGUI_FORM_EDIT_TSD_H
#include <mgui/formedittsd.h>
#endif

#ifndef INC_MGUI_DLGGETOBJECTFILTER_H
#include <mgui/dlggetobjectfilter.h>
#endif

class FILEPATHLIST;
namespace MGUI {

class FORM_LINKTOTILESET : public MGUI::FORM_COMPOSITE {
	public:
		FORM_LINKTOTILESET ();
		virtual ~FORM_LINKTOTILESET ();

		static ERRVALUE RunJob (const FILEPATH& path);

		//! Create form
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane		//!< Parent pane
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		enum COL {
			COL_TilesetName = 0,
			COL_Type,
			COL_MinLevel,
			COL_MaxLevel,
			COL_TypeID,
			COL_TilesetPath,
			COL_COUNT
			};

		enum CMDID {
			CMDID_JobSave = 1,
			CMDID_JobQueue,
			CMDID_FileRun,
			CMDID_ItemsAdd,
			CMDID_ItemsRemoveSelected,
			CMDID_ItemsRemoveAll,
			CMDID_COUNT
			};

		enum TILESET {
			TILESET_GoogleMaps = 1,
			TILESET_GoogleEarth = 2,
			TILESET_BingMaps = 3,
			TILESET_NASAWorldWind = 4
			};

		class FOLDER_FILTER : public MGUI::DLG_GETOBJECT_FILTER {
			public:
				FOLDER_FILTER ();
				virtual ~FOLDER_FILTER ();
			private:
				virtual bool v_IsFolderSelectable (
					const FILEPATH& FolderPath,			//!< Folder to check
					int position						//!< Position of the file in the set for DLG_GETOBJECT_SET.
					) const;
			};

		MGUI::CTRL_TOOLBAR m_ToolBar;
		MGUI::FORM_LIST m_TSDList;
		MGUI::CTRL_TOGGLEBUTTON m_TglAutoCreateTSD;
		MGUI::CTRL_TOGGLEBUTTON m_TglUseLowestLevel;
		MGUI::CTRL_TOGGLEBUTTON m_TglMakeTileMap;
		MGUI::FORM_EDIT_TSD m_FormEditTSD;
		MGUI::CTRL_PUSHBUTTON m_SaveBtn;
		MGUI::LAYOUT_BOOK m_Book;
		MGUI::LAYOUT_PAGE m_PageAuto;
		MGUI::LAYOUT_PAGE m_PageManual;

		void CreateJob (bool held);
		void OnRunJob ();
		void OnSaveJob ();

		ERRVALUE AddSrcItems ();
		ERRVALUE DoCmdRun ();
		ERRVALUE GetOutput (FILEPATHLIST& TSDPaths);
		bool OnClick (int itemnum, int fieldnum, const MGUI::POINT&, MGUI::POINTERBUTTON, MGUI::KEYSTATE);
		void OnPageInitAuto (MGUI::LAYOUT_PAGE& page);
		void OnPageInitManual (MGUI::LAYOUT_PAGE& page);
		void OnSaveTSD();
		void OnToolBar(MGUI::ID cmdid, bool state);
		void OnTglAutoCreate ();
		void RemoveSrcItemsSelected();
		void RemoveAllSrcItems();
		static ERRVALUE Run (
			const FILEPATH& FolderPath,
			const FILEPATH& TSDPath,
			TILESET Type,
			INT32 MinLevel,
			INT32 MaxLevel,
			bool ScanBottomOnly,
			bool MakeTileMap
			);
		void SetType (int itemnum, TILESET type);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}	// End of MGUI namespace

#endif
