/**
 * \file mgui/formradiobuttons.h
 * \brief MGUI::FORM_RADIOBUTTONS definition
 *
 * \if NODOC
 * $Id: formradiobuttons.h_v 1.3 2011/08/25 21:16:23 mju Exp $
 *
 * $Log: formradiobuttons.h_v $
 * Revision 1.3  2011/08/25 21:16:23  mju
 * Add setToggleButtonStyle.
 *
 * Revision 1.2  2011/08/25 19:48:57  mju
 * Add getSelectedIcon.
 *
 * Revision 1.1  2011/08/25 17:37:50  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORMRADIOBUTTONS_H
#define	INC_MGUI_FORMRADIOBUTTONS_H

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

namespace MGUI {

//===================================================================================================================
//! Form containing set of radio buttons, with enforcement of radio behavior.
class FORM_RADIOBUTTONS : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		FORM_RADIOBUTTONS ();

		//! Destructor.
		virtual ~FORM_RADIOBUTTONS ();

		//! Add icon button with ICON.
		//! Button may be added before or after Create.
		void AddButton (
			ID id,											//!< ID to attach to button
			const ICON& icon,								//!< Icon
			const MISTRING& tooltip						//!< ToolTip string
			);

		//! Add icon button with ICONID.
		//! Button may be added before or after Create.
		void AddButton (
			ID id,											//!< ID to attach to button
			ICONID iconid,									//!< Icon ID
			const MISTRING& tooltip,					//!< ToolTip string
			UINT16 iconsize = 0							//!< Icon size, 0 for default
			);

		//! Add button with text label (no icon).
		//! Button may be added before or after Create.
		void AddButton (
			ID id,											//!< ID to attach to button
			const MISTRING& string						//!< String for button
			);

		//! Create form with buttons in single column or row.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			MGUI::LAYOUT_ORIENTATION orientation = MGUI::LAYOUT_ORIENTATION_Vertical,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			int Spacing = 4,								//!< Spacing between buttons
			int Margin = 0									//!< Margin around form
			);

		//! Create form with buttons in multiple rows or columns.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			MGUI::LAYOUT_ORIENTATION orientation,	//!< Primary orientation
			int NumPerRowOrColumn,						//!< Number of buttons per row (for Horizontal) or column (for Vertical)
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedSize,
			int Spacing = 4,								//!< Spacing between buttons
			int Margin = 0									//!< Margin around form
			);

		//! Get ICON for button.
		const ICON& GetIcon (
			ID id												//!< ID of button
			) const;

		// Get ICON for current selection.
		const ICON& GetSelectedIcon () const;

		//! Get ID of currently selected button.
		//! @return ID or -1 if none selected.
		ID GetSelectedID (
			) const;

		//! Set delegate to all when user makes selection.
		void SetDelegateOnSelection (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set whether individual button is enabled or not.
		void SetEnabledID (
			ID id,											//!< ID of button to enable/disable
			bool enabled = true							//!< 'true' to enable, 'false' to disable
			);

		//! Select button using ID.
		//! If button does not exist then no button will appear selected
		void SetSelectedID (
			ID id												//!< ID of button to select
			);

		//! Set togglebutton style.
		//! Must be done before Create.
		void SetToggleButtonStyle (
			CTRL_TOGGLEBUTTON::STYLE ToggleStyle	//!< ToggleButton style
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_RADIOBUTTONS (const FORM_RADIOBUTTONS&);
		FORM_RADIOBUTTONS& operator= (const FORM_RADIOBUTTONS&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORMRADIOBUTTONS_H
