/**
 * \file mgui/formrasterfilterparms.h
 *
 * \if NODOC
 * $Id: formrasterfilterparms.h_v 1.3 2012/05/29 13:53:55 mju Exp $
 *
 * $Log: formrasterfilterparms.h_v $
 * Revision 1.3  2012/05/29 13:53:55  mju
 * Add thisclass typedef.
 *
 * Revision 1.2  2011/09/12 16:55:09  mju
 * Add setDelegateOnChangeValue.
 *
 * Revision 1.1  2007/11/19 22:04:11  bcolombe
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_FORMRASTERFILTERPARMS_H
#define	INC_MGUI_FORMRASTERFILTERPARMS_H

#ifndef INC_MI32_RASTERFILTERPARMS_H
#include <mi32/rasterfilterparms.h>
#endif

#ifndef  INC_MI32_MIMATRIX_H
#include <mi32/mimatrix.h>
#endif

#ifndef INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

#ifndef INC_MGUI_SLIDER_H
#include <mgui/slider.h>
#endif

namespace MGUI {

//! Form for spatial filter gui controls
class FORM_RASTER_FILTER_PARMS : public MGUI::FORM_COMPOSITE {
	public:

		FORM_RASTER_FILTER_PARMS ();

		~FORM_RASTER_FILTER_PARMS ();

		//! Create Form
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Get settings as RASTER_FILTER_PARMS
		void GetValues (RASTER_FILTER_PARMS& result);

		//! Set delegate to call when any value changes.
		void SetDelegateOnChangeValue (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnChangeValue = delegate; }

		//! Set initial values from RASTER_FILTER_PARMS
		void SetValues (const RASTER_FILTER_PARMS& values); 

	private:	
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_RASTER_FILTER_PARMS THISCLASS;

		RVC::SPATIALFILTER::FILTERTYPE m_FilterType;			// Filter type
		RASTER_FILTER_PARMS::GRADIENTTYPE m_GradientType;	// Gradient type
		UINT32 m_NumRows;			// Size of Kernels
		UINT32 m_NumColumns;		// Size of Kernels
		MIMATRIX m_MatrixA;		// Kernel A
		MIMATRIX m_MatrixB;		// Kernel B

		RASTER_FILTER_PARMS m_RasterFilterParms;	//Settings

		//mgui:
		MGUI::CTRL_TOGGLEBUTTON m_EnableToggle;
		MGUI::CTRL_TEXT m_TextBox;
		MGUI::CTRL_PUSHBUTTON m_EditKernel;
		MGUI::FORM_COMBOBOX m_ClassComboBox;
		MGUI::FORM_COMBOBOX m_TypeComboBox;
		MGUI::FORM_COMBOBOX m_SizeComboBox;
		MGUI::FORM_SLIDER *m_AmountSlider;
		MGUI::FORM_SLIDER *m_ParamASlider;
		MGUI::FORM_SLIDER *m_ParamBSlider;
		DELEGATE_VOID_NOPARMS m_DelegateOnChangeValue;

		void NotifyChange ();
		void OnClassChanged ();	//delegate for class combobox
		void OnEditKernel ();	//delegate for setting kernel values
		void OnSizeChanged ();	//delegate for kernel size changing
		void OnTypeChanged ();	//delegate for type combobox

		#endif
	};

}	// End namespace MGUI

#endif
