/**
 * \file mgui/keycode.h
 * \brief KEYCODE definitions
 *
 * \if NODOC
 * $Id: keycode.h_v 1.8 2008/03/14 15:37:53 mju Exp $
 *
 * $Log: keycode.h_v $
 * Revision 1.8  2008/03/14 15:37:53  mju
 * Use mgui base hdr.
 *
 * Revision 1.7  2007/12/06 16:16:47  mju
 * Add keypad codes and arrange so documentation works better.
 *
 * Revision 1.6  2007/06/28 15:23:03  dwilliss
 * doxygen help
 *
 * Revision 1.5  2005/03/31 16:57:08  fileserver!dwilliss
 * Rename one of our types to MIUNICODE because it conflicted with a Microsoft #define
 *
 * Revision 1.4  2003/10/03 19:58:19  linux32build!build
 * Doxygen
 *
 * Revision 1.3  2003/09/25 21:25:01  dwilliss
 * Don't capitalize enum any more. Genitor needed it, doxygen doesn't like it.
 *
 * Revision 1.2  2003/09/15 13:49:32  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.1  2003/01/07 18:26:16  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_KEYCODE_H
#define	INC_MGUI_KEYCODE_H

#ifndef	INC_MGUI_BASE_H
#include <mgui/base.h>
#endif

#ifdef X_NATIVE
#include <X11/keysym.h>
#define	KC_ITEM(w,x)			(x)
#define	KC_ITEMENHANCED(w,x)	(x)
#else
#define	KC_ENHANCEBITS			0x8000
#define	KC_ITEM(w,x)			(w)
#define	KC_ITEMENHANCED(w,x)	(w|0x8000)
#endif

namespace MGUI {

//! Keycode enumeration, non-enumerated values are equivalent to MIUNICODE.
//! Note that there are special codes for "keypad" keys.  Depending on the
//! application these should be processed the same as the equivalent non-keypad key.
enum KEYCODE {
	// TTY keys
	KEYCODE_BackSpace	= KC_ITEM(VK_BACK,XK_BackSpace),
	KEYCODE_Delete		= KC_ITEM(VK_DELETE,XK_Delete),
	KEYCODE_Tab			= KC_ITEM(VK_TAB,XK_Tab),
	KEYCODE_Clear		= KC_ITEM(VK_CLEAR,XK_Clear),
	KEYCODE_Return		= KC_ITEM(VK_RETURN,XK_Return),
	KEYCODE_Escape		= KC_ITEM(VK_ESCAPE,XK_Escape),
	KEYCODE_Pause		= KC_ITEM(VK_PAUSE,XK_Pause),

	// Misc
	KEYCODE_Select		= KC_ITEM(VK_SELECT,XK_Select),
	KEYCODE_Help		= KC_ITEM(VK_HELP,XK_Help),
	KEYCODE_Space		= KC_ITEM(VK_SPACE,XK_space),
	KEYCODE_Insert		= KC_ITEM(VK_INSERT,XK_Insert),

	// Motion keys
	KEYCODE_Home		= KC_ITEM(VK_HOME,XK_Home),
	KEYCODE_End			= KC_ITEM(VK_END,XK_End),
	KEYCODE_Left		= KC_ITEM(VK_LEFT,XK_Left),
	KEYCODE_Right		= KC_ITEM(VK_RIGHT,XK_Right),
	KEYCODE_Up			= KC_ITEM(VK_UP,XK_Up),
	KEYCODE_Down		= KC_ITEM(VK_DOWN,XK_Down),
	KEYCODE_PageUp		= KC_ITEM(VK_PRIOR,XK_Prior),
	KEYCODE_PageDown	= KC_ITEM(VK_NEXT,XK_Next),

	// Keypad keys
	KEYCODE_KP_Enter		= KC_ITEMENHANCED(VK_RETURN,XK_KP_Enter),
	KEYCODE_KP_Home		= KC_ITEMENHANCED(VK_HOME,XK_KP_Home),
	KEYCODE_KP_Left		= KC_ITEMENHANCED(VK_LEFT,XK_KP_Left),
	KEYCODE_KP_Up			= KC_ITEMENHANCED(VK_UP,XK_KP_Up),
	KEYCODE_KP_Right		= KC_ITEMENHANCED(VK_RIGHT,XK_KP_Right),
	KEYCODE_KP_Down		= KC_ITEMENHANCED(VK_DOWN,XK_KP_Down),
	KEYCODE_KP_PageUp		= KC_ITEMENHANCED(VK_PRIOR,XK_KP_Page_Up),
	KEYCODE_KP_PageDown	= KC_ITEMENHANCED(VK_NEXT,XK_KP_Page_Down),
	KEYCODE_KP_End			= KC_ITEMENHANCED(VK_END,XK_KP_End),
	KEYCODE_KP_Insert		= KC_ITEMENHANCED(VK_INSERT,XK_KP_Insert),
	KEYCODE_KP_Delete		= KC_ITEMENHANCED(VK_DELETE,XK_KP_Delete),
	KEYCODE_KP_Multiply	= KC_ITEM(VK_MULTIPLY,XK_KP_Multiply),
	KEYCODE_KP_Add			= KC_ITEM(VK_ADD,XK_KP_Add),
	KEYCODE_KP_Subtract	= KC_ITEM(VK_SUBTRACT,XK_KP_Subtract),
	KEYCODE_KP_Decimal	= KC_ITEM(VK_DECIMAL,XK_KP_Decimal),
	KEYCODE_KP_Divide		= KC_ITEM(VK_DIVIDE,XK_KP_Divide),

	KEYCODE_KP_0		= KC_ITEM(VK_NUMPAD0,XK_KP_0),
	KEYCODE_KP_1		= KC_ITEM(VK_NUMPAD1,XK_KP_1),
	KEYCODE_KP_2		= KC_ITEM(VK_NUMPAD2,XK_KP_2),
	KEYCODE_KP_3		= KC_ITEM(VK_NUMPAD3,XK_KP_3),
	KEYCODE_KP_4		= KC_ITEM(VK_NUMPAD4,XK_KP_4),
	KEYCODE_KP_5		= KC_ITEM(VK_NUMPAD5,XK_KP_5),
	KEYCODE_KP_6		= KC_ITEM(VK_NUMPAD6,XK_KP_6),
	KEYCODE_KP_7		= KC_ITEM(VK_NUMPAD7,XK_KP_7),
	KEYCODE_KP_8		= KC_ITEM(VK_NUMPAD8,XK_KP_8),
	KEYCODE_KP_9		= KC_ITEM(VK_NUMPAD9,XK_KP_9),

	// Function keys
	KEYCODE_F1		= KC_ITEM(VK_F1,XK_F1),
	KEYCODE_F2		= KC_ITEM(VK_F2,XK_F2),
	KEYCODE_F3		= KC_ITEM(VK_F3,XK_F3),
	KEYCODE_F4		= KC_ITEM(VK_F4,XK_F4),
	KEYCODE_F5		= KC_ITEM(VK_F5,XK_F5),
	KEYCODE_F6		= KC_ITEM(VK_F6,XK_F6),
	KEYCODE_F7		= KC_ITEM(VK_F7,XK_F7),
	KEYCODE_F8		= KC_ITEM(VK_F8,XK_F8),
	KEYCODE_F9		= KC_ITEM(VK_F9,XK_F9),
	KEYCODE_F10		= KC_ITEM(VK_F10,XK_F10),
	KEYCODE_F11		= KC_ITEM(VK_F11,XK_F11),
	KEYCODE_F12		= KC_ITEM(VK_F12,XK_F12),
	KEYCODE_F13		= KC_ITEM(VK_F13,XK_F13),
	KEYCODE_F14		= KC_ITEM(VK_F14,XK_F14),
	KEYCODE_F15		= KC_ITEM(VK_F15,XK_F15),
	KEYCODE_F16		= KC_ITEM(VK_F16,XK_F16),
	KEYCODE_F17		= KC_ITEM(VK_F17,XK_F17),
	KEYCODE_F18		= KC_ITEM(VK_F18,XK_F18),
	KEYCODE_F19		= KC_ITEM(VK_F19,XK_F19),
	KEYCODE_F20		= KC_ITEM(VK_F20,XK_F20),
	KEYCODE_F21		= KC_ITEM(VK_F21,XK_F21),
	KEYCODE_F22		= KC_ITEM(VK_F22,XK_F22),
	KEYCODE_F23		= KC_ITEM(VK_F23,XK_F23),
	KEYCODE_F24		= KC_ITEM(VK_F24,XK_F24),

	};

//! Operator allowing assignment from MIUNICODE.
//!inline KEYCODE operator= (MIUNICODE rhs) { return (static_cast<KEYCODE>(rhs)); }

}	// End namespace MGUI

#undef	KC_ITEM

#endif	// INC_MGUI_KEYCODE_H
