/**
 * \file mi32/avi.h
 * \brief Header for MicroImages AVI functions.
 *
 * \if NODOC
 * $Id: avi.h_v 1.8 2011/10/21 20:41:26 mju Exp $
 *
 * $Log: avi.h_v $
 * Revision 1.8  2011/10/21 20:41:26  mju
 * Remove redundant hdr.
 *
 * Revision 1.7  2007/03/21 15:07:41  honerkamp
 * Changed movieencstart to use FILEPATH
 *
 * Revision 1.1  1999/03/31  16:43:16  vdronov
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_AVI_H
#define	INC_MI32_AVI_H

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

/*----------------------------------------------------------------------------*/
/*		Constants and structures																*/
/*----------------------------------------------------------------------------*/

struct MAVIPARMENCODE {
	INT32 hsize;								//!<  Frame horizontal size 
	INT32 vsize;								//!<  Frame vertical size 
	UINT8 frame_rate;							//!<  Frame rate (FRAMERATE_...) 
	};

/*----------------------------------------------------------------------------*/
/*		Macros and functions prototypes														*/
/*----------------------------------------------------------------------------*/

#if defined(__cplusplus)
extern "C" {
#endif

//! Checks to see if OS supports AVI recording.
//!
//! Returns non-0 value if current OS support AVI recording, otherwise 0.
int MaviEncodeIsAvailable ();

//! Add frames to movie.
int MaviEncodeFrame (
	void *handle, 
	void *rgbdata
	);

//! Gets handle for process that creates movie in the filename file, parameters given in MAVIPARMENCODE struct.
int MaviEncodeStart (
	void **handle, 
	const FILEPATH &filepath, 
	MAVIPARMENCODE *parm
	);

//! Closes movie creation process and closes corresponding file.
void MaviEncodeStop (
	void **handle
	);

#if defined(__cplusplus)
}
#endif

#endif	//!<  ifndef INC_MI32_AVI_H 
