/**
 * \file fileaccess.h <mi32/fileaccess.h>
 * \brief File access mode definitions
 *
 * \if NODOC
 * $Id: fileaccess.h_v 1.4 2006/07/17 20:10:36 scowan Exp $
 *
 * $Log: fileaccess.h_v $
 * Revision 1.4  2006/07/17 20:10:36  scowan
 * Addec create no cache flag.
 *
 * Revision 1.3  2006/07/14 15:00:36  scowan
 * Added no cache flags.
 *
 * Revision 1.2  2005/07/20 14:16:40  mju
 * Move share modes here and add share_auto mode.
 *
 * Revision 1.1  2005/07/19 17:41:02  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_FILEACCESS_H
#define	INC_MI32_FILEACCESS_H

//! File access modes.
enum FILE_ACCESS {
	FILE_ACCESS_Read =					0x01,		//!< Open for reading
	FILE_ACCESS_Write =					0x02,		//!< Open for writing, create if does not exist, retains contents if exists
	FILE_ACCESS_ReadWrite =				0x03,		//!< Open for both reading and writing, will create if does not exist
	FILE_ACCESS_Create =					0x04,		//!< Create or truncate
	FILE_ACCESS_Append =					0x08,		//!< Append to file, may limit writing to end of file
	FILE_ACCESS_ReadNoCache = 			0x11,		//!< Open for reading, OS cache is disabled, buffer must be aligned
	FILE_ACCESS_WriteNoCache = 		0x12,		//!< Open for writing, create if does not exist, retains contents if exists, OS cache is disabled, buffer must be aligned
	FILE_ACCESS_ReadWriteNoCache =	0x13,		//!< Open for both reading and writing, will create if does not exist
	FILE_ACCESS_CreateNoCache =		0x14		//!< Create or truncate, OS cache is disabled, buffer must be aligned
	};

//! File sharing modes.
enum FILE_SHARE {
	FILE_SHARE_Any =			0x00,
	FILE_SHARE_Exclusive =	0x01,		//!< This process has exclusive access
	FILE_SHARE_ReadOnly =	0x02,		//!< File can be shared as read-only
	FILE_SHARE_Auto =			0x04		//!< Determine share mode as Read if FILE_ACCESS_Read and Exclusive if FILE_ACCESS_Write
	};

#endif	// INC_MI32_FILEACCESS_H
