/**
 * \file mi32/lidarclassifiercs.h
 * \brief LIDAR::CLASSIFIER_CLIMBSLIDE definitions
 *
 * \if NODOC
 * $Id: lidarclassifiercs.h_v 1.2 2012/10/03 14:58:12 mju Exp $
 *
 * $Log: lidarclassifiercs.h_v $
 * Revision 1.2  2012/10/03 14:58:12  mju
 * Add setting for saving test results.
 *
 * Revision 1.1  2012/10/02 14:17:10  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_LIDARCLASSIFIERCS_H
#define	INC_MI32_LIDARCLASSIFIERCS_H

#ifndef	INC_MI32_LIDARCLASSIFIER_H
#include <mi32/lidarclassifier.h>
#endif

#ifndef	INC_MI32_FILEPATH_H
#include <mi32/filepath.h>
#endif

namespace LIDAR {

//=====================================================================================================================

//! Climbing and Sliding Classifier to classify points as "Ground" or "Unclassified" (non-ground).
class CLASSIFIER_CLIMBSLIDE : public CLASSIFIER {
	public:

		//! Constructor.
		CLASSIFIER_CLIMBSLIDE ();

		//! Destructor.
		~CLASSIFIER_CLIMBSLIDE ();

		//! Get grid interval.
		double GetGridInterval () const
			{ return (m_GridInterval); }

		//! Get seed interval.
		double GetSeedInterval () const
			{ return (m_SeedInterval); }

		//! Get general slope value.
		double GetSlopeGeneral () const
			{ return (m_SlopeGeneral); }

		//! Get slope increment value.
		double GetSlopeIncrement () const
			{ return (m_SlopeIncrement); }

		//! Get maximum slope value.
		double GetSlopeMaximum () const
			{ return (m_SlopeMaximum); }

		//! Run classifier.
		ERRVALUE Run (
			LIDAR::LAS::DATASET& dataset,
			const DRECT2D& extents
			);

		//! Set grid interval.
		void SetGridInterval (
			double GridInterval				//!< Grid interval, typically 1-3X point spacing
			) { m_GridInterval = GridInterval; }

		//! Set seed interval.
		void SetSeedInterval (
			double SeedInterval				//!< Seed interval, typically larger than largest building size
			) { m_SeedInterval = SeedInterval; }

		//! Set general slope value.
		void SetSlopeGeneral (
			double SlopeGeneral				//!< General slope, (.1 typical)
			) { m_SlopeGeneral = SlopeGeneral; }

		//! Set slope increment value.
		void SetSlopeIncrement (
			double SlopeIncrement			//!< Slope increment, (.05 - .15 typical)
			) { m_SlopeIncrement = SlopeIncrement; }

		//! Set maximum slope value.
		void SetSlopeMaximum (
			double SlopeMaximum				//!< Maximum slope, (< .60, .40 typical)
			) { m_SlopeMaximum = SlopeMaximum; }

	#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Set test path.
		void SetTestPath (
			const FILEPATH& filepath
			) { m_SaveTest = true; m_FilePathTest = filepath; }
	#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		enum OUTPUT {
			OUTPUT_Ground = 0,
			OUTPUT_Unclassified = 1
			};

		class ENGINE;

		double m_GridInterval;
		double m_SeedInterval;
		double m_SlopeGeneral;
		double m_SlopeIncrement;
		double m_SlopeMaximum;
		FILEPATH m_FilePathTest;
		bool m_SaveTest;

		// Overrides.
		virtual double v_GetMargin () const;
		virtual UINT64 v_GetMaxPointsDesired () const;
		virtual void v_IniRead (INIHANDLE inihnd);
		virtual void v_IniWrite (INIHANDLE inihnd) const;
		virtual void v_JobGetValues (MIJOB& mijob) const;
		virtual ERRVALUE v_JobRead (const MIJOB& mijob);
		virtual void v_Reset ();
		#endif
	};

//=====================================================================================================================

}	// End namespace LIDAR

#endif	// INC_MI32_LIDARCLASSIFIERCS_H
