/******************************************************************************
 * \file mi32/mg2mem16.h
 * \brief A RAM-based implementation of MGD::DEVICE for 16-bit RGB visuals
 *
 * \if NODOC
 * $Log: mg2mem16.h_v $
 * Revision 1.10  2011/10/26 17:57:54  mju
 * Frame buffer.
 *
 * Revision 1.9  2005/09/15 21:36:58  mju
 * Fix unref parm warning.
 *
 * Revision 1.7  2003/12/15 23:35:53  dwilliss
 * Diverging for 7.0
 * Made changes necessary for graphical data tips
 *
 * Revision 1.3  2002/10/17 15:22:13  mju
 * COLORSPACE enum value changed.
 *
 * Revision 1.1  2002/01/29 15:19:36  dwilliss
 * Initial revision
 * \endif
**/

#ifndef INC_MI32_MG2MEM16_H
#define INC_MI32_MG2MEM16_H

#ifndef INC_MI32_MGD2_H
#include <mi32/mgd2.h>
#endif


namespace MGD {


class CONTEXT_MEM16;

class DEVICE_MEM16 : public MGD::DEVICE {

	public:

		//! constructor
		DEVICE_MEM16();

		//! Destructor
		virtual ~DEVICE_MEM16();

		//! Note: Can use only use COLORSPACE_Packed16_X1_R5_G5_B5, and _Packed16_X1_R5_G5_B5 for now,
		//!   but _Indexed16Bit or _Gray16Bit are planned.

		ERRVALUE Create (
			INT32 height,
			INT32 width,
			COLORSPACE colorspace = COLORSPACE_Packed16_X1_R5_G5_B5
			);

		
	protected:

		UINT32 m_BytesPerLine;
		COLORSPACE m_ColorSpace;	

		//! Fill a horizontal span.
		//!
		//! Many of the default method implementations call this to do all the work
		//! Must be protected: Derived classes must be able to call it.
		virtual ERRVALUE v_FillSpan (
			MGD::CONTEXT* vgc,
			INT32 x1,
			INT32 x2,
			INT32 y
			);

		virtual CAPABILITY v_GetCapabilities (
			) const;

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		// Unimplemented.
		DEVICE_MEM16 (const DEVICE_MEM16&);

		UINT8 *m_buf;

		friend class MGD::CONTEXT_MEM16;
		
		virtual CONTEXT* v_CreateContext ();
		virtual COLORSPACE v_GetColorSpace () const;
		virtual UINT8* v_GetImagePointer (INT32 x, INT32 y, INT32 len, UINT16 FrameBuffer);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

		
class CONTEXT_MEM16 : public MGD::CONTEXT {
	public:
		// Note, default Constructor is protected

		virtual ~CONTEXT_MEM16();

	protected:

		//! Default Constructor 
		CONTEXT_MEM16 (
			MGD::DEVICE_MEM16* disp
			);

		virtual ERRVALUE v_PutImageSpanDirect (
			INT32 x,
			INT32 y,
			INT32 len,
			const void* data,
			const UINT8* mask
			);

	private:
   	#ifndef GENERATING_DOXYGEN_OUTPUT

		MGD::DEVICE_MEM16* m_DevMem16;

		//! Copy constructor private and unimplemented
		CONTEXT_MEM16(const CONTEXT_MEM16&);
	
		friend class MGD::DEVICE_MEM16;	//!< Only MGD_DEVICE::CreateGC() can call our constructor
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


}	//! End of namespace

#endif	// INC_MI32_MG2MEM16_H

