/**
 * \file mi32/mprint.h
 * \brief Printing subsystem definitions.
 *
 * \if NODOC
 * $Id: mprint.h_v 1.27 2012/10/26 14:52:19 mju Exp $
 *
 * $Log: mprint.h_v $
 * Revision 1.27  2012/10/26 14:52:19  mju
 * Remove mprintMainInit/Stop as don't do anything.
 *
 * Revision 1.26  2011/11/02 21:35:42  mju
 * Remove redundant hdr.
 *
 * Revision 1.24  2008/01/18 21:14:39  mju
 * Add dft flags for printLayout.
 *
 * Revision 1.22  2007/04/09 17:47:34  dwilliss
 * Changed printer gamma structure
 *
 * Revision 1.9  1999/04/08 21:01:31  mju
 * Use correct inclusion guards.
 *
 * \endif
**/

#ifndef INC_MI32_MPRINT_H
#define INC_MI32_MPRINT_H

#ifndef	INC_MI32_PRINTGAMMA_H
#include <mi32/printgamma.h>
#endif

#ifndef	INC_GRE_BASE_H
#include <gre/base.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
struct LAYOUTPARMS;
namespace MGD {
	class DEVICE;
	}
#endif


#define MPRINT_NoDialog				0x00000001	//!<  Skip the setup dialog unless not enough info 
#define MPRINT_DeleteWhenDone		0x00000002	//!<  Delete print raster when done 
#define MPRINT_Snapshot				0x00000004
#define MPRINT_TestStrips			0x00000008	//!<  Only when called internally to print test strips 


extern "C" {

//!	Given a layout, create a print view and print it.
int MprintMainPrintLayout (
	MDLGPARENT parent,	 						//!< Parent for setup and status dialogs
	GRE::LAYOUT *layout, 						//!< The layout to print.
	UINT32 flags = 0								//!< MPRINT_NoDoalog
	);

//! Parse command line arguments and decide what to do.
int MprintMain (
	MDLGPARENT parent, 
	int argc, 
	const char **argv
	);

//! Setup ICM proofing on the MGD device. 
//! If LAYOUTPARMS.bProofOnScreen is false, it actually turns off proofing.
ERRVALUE MprintSetupProofICM (
	MGD::DEVICE*,
	const LAYOUTPARMS&
	);

}	// Extern "C"


#endif	// INC_MI32_MPRINT_H 
