/**
 * \file mi32/platformsupport.h
 * \brief Define constants to test whether platform supports particular capabilities
 *
 * \if NODOC
 * $Id: platformsupport.h_v 1.22 2010/08/27 20:55:38 bcolombe Exp $
 *
 * $Log: platformsupport.h_v $
 * Revision 1.22  2010/08/27 20:55:38  bcolombe
 * win64 supports ecw now
 *
 * Revision 1.21  2010/03/25 17:20:26  bcolombe
 * Enable ECW on mac
 *
 * Revision 1.20  2008/11/25 16:12:58  scowan
 * Enabled 64bit Windows support for dgn.
 *
 * Revision 1.19  2008/11/19 23:06:03  scowan
 * *** empty log message ***
 *
 * Revision 1.18  2008/11/19 22:34:45  scowan
 * Added mac for dwg support.
 *
 * Revision 1.17  2008/10/30 20:22:05  scowan
 * Disabled dgn for 64bit Windows.
 *
 * Revision 1.16  2008/10/23 21:24:35  scowan
 * Removed unnecessary help id include file.
 *
 * Revision 1.15  2008/10/23 14:53:51  scowan
 * Updated dwg and added dgn defines.
 *
 * Revision 1.14  2008/10/16 16:29:36  bcolombe
 * ECW doesn't work on mac
 *
 * Revision 1.13  2008/09/30 17:39:48  scowan
 * Added oracle occi define.
 *
 * Revision 1.12  2008/09/10 23:01:57  scowan
 * Added oracle occi define.
 *
 * Revision 1.11  2008/08/21 16:59:47  mju
 * Enable mrsid for win64.
 *
 * Revision 1.10  2008/08/21 14:22:03  mju
 * Allow mrsid for mac.
 *
 * Revision 1.9  2008/08/21 14:16:14  mju
 * Allow mrsid for macx86_64.
 *
 * Revision 1.8  2008/04/22 21:35:31  mju
 * Add platform_supports_ecw_export.
 *
 * Revision 1.7  2008/04/22 21:21:48  mju
 * Add ecw.
 *
 * Revision 1.6  2007/11/14 22:05:05  dwilliss
 * Added DWG define
 *
 * Revision 1.5  2007/11/14 20:18:07  dwilliss
 * MrSID not supported on Mac Intel x86_64
 *
 * Revision 1.4  2006/11/12 22:50:41  bcolombe
 * Garmin change
 *
 * Revision 1.3  2006/11/10 23:03:26  bcolombe
 * Changed Garmin support
 *
 * Revision 1.2  2006/11/10 20:32:28  bcolombe
 * Added Garmin Support
 *
 * Revision 1.1  2006/10/19 21:24:36  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_PLATFORMSUPPORT_H
#define	INC_MI32_PLATFORMSUPPORT_H

// MrSID
#if defined(WIN32) || (defined(MAC_OSX) /*&& !defined(__x86_64__)*/)// || (defined(LINUX_INTEL) && !defined(LONGS_ARE_64_BIT))
	#define	PLATFORM_SUPPORTS_MRSID
#endif

// ECW
#if defined(WIN32) || defined(MAC_OSX) // || (defined(LINUX_INTEL) && !defined(LONGS_ARE_64_BIT))
	#define	PLATFORM_SUPPORTS_ECW
#endif

// ECW_EXPORT
#if defined(WIN32) // || defined(MAC_OSX) // || (defined(LINUX_INTEL) && !defined(LONGS_ARE_64_BIT))
	#define	PLATFORM_SUPPORTS_ECW_EXPORT
#endif

// DWG through the OpenDesign alliance libraries
#if defined(WIN32) || defined(WIN64) || defined(MAC_OSX)
	#define PLATFORM_SUPPORTS_DWG
#endif

// DGN through the OpenDesign alliance libraries
//#if defined(WIN32) || defined(WIN64)	// || (defined(MAC_OSX) && !defined(__x86_64__))
#if defined(WIN32) || defined(WIN64)
	#define PLATFORM_SUPPORTS_DGN
#endif

// Garmin
#if defined(WIN32) || defined(WIN64)
	#define PLATFORM_SUPPORTS_GARMIN
#endif

// MySQL
#if defined(WIN32) && !defined(WIN64)
	#define PLATFORM_SUPPORTS_MYSQL
#endif

// PostGRES
#if (defined(WIN32) && !defined(WIN64)) || defined(MAC_OSX)
	#define PLATFORM_SUPPORTS_POSTGRES
#endif

// Oracle OCCI
#if defined(WIN32) || defined(WIN64) // || defined(MAC_OSX)
	#define PLATFORM_SUPPORTS_ORACLE_OCCI
#endif

#endif	// INC_MI32_PLATFORMSUPPORT_H
