/**
 * \file rvc/opviewshed.h
 * \brief Definitions for RVC::OP::VIEWSHED class.
 *
 * \if NODOC
 * $Id: opviewshed.h_v 1.4 2012/06/22 16:11:18 mju Exp $
 *
 * $Log: opviewshed.h_v $
 * Revision 1.4  2012/06/22 16:11:18  mju
 * Add hdrs so compiles if first.
 *
 * Revision 1.3  2011/09/02 13:07:40  mju
 * Exclude macro from dox.
 *
 * Revision 1.2  2006/08/30 19:21:15  scowan
 * Changed flags to be the enum.
 *
 * Revision 1.1  2006/08/21 19:02:19  jjohnson
 * Initial revision
 *
 * \endif
**/

#ifndef  INC_RVC_OPVIEWSHED_H
#define  INC_RVC_OPVIEWSHED_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#ifndef	INC_MEMORY_H
#define	INC_MEMORY_H
#include <memory.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace RVC {
	class OBJITEM;
	}
#endif


namespace RVC {
namespace OP {

//! Class for creating viewshed rasters.
class VIEWSHED {
	public:
		enum FLAGS {
			FLAG_None = 0x0000,
			FLAG_EarthCurvature = 0x0001,
			FLAG_InvertTransparency = 0x0002
			};

		struct VIEWPOINTINFO {
			DPOINT3D m_Point;
			double m_Height;				//!< height of view  
			double m_Distance;			//!< max distance for point to be visible from observer 
			DRANGE m_HrzAngle;			//!< View angle horizontal limits (0 - 360 degrees)
			DRANGE m_VrtAngle;			//!< View angle vertical limits (-90 - 90 degrees)
			VIEWPOINTINFO (
				) { memset(this, 0, sizeof(*this)); }
			};

		VIEWSHED () { }

		static ERRVALUE Compute (
			const RVC::OBJITEM& SrcItem,				//!< Input raster
			RVC::OBJITEM& DestItem,						//!< Output raster
			DOUBLE_ARRAY<RVC::OP::VIEWSHED::VIEWPOINTINFO>& ViewpointInfo,  //!< View point information				
			double zscale,			  						//!< Z scale
			UINT8 enviewpts, 		  						//!< number of view points enough for cell to be "visible"
			FLAGS flags = FLAG_None						//!< Optional flags: FLAG_EarthCurvature and FLAG_InvertTransparency
			);

		static ERRVALUE Compute (
			const RVC::OBJITEM& SrcItem,		//!< Input raster
			RVC::OBJITEM& DestItem,				//!< Output raster
			const LPOINT2D *viewpts,			//!< viewpoints list
			double height,			  				//!< height of view
			double zscale,			  				//!< Z scale
			UINT8 numviewpts, 					//!< number of viewpoints
			UINT8 enviewpts, 		  				//!< number of view points enough for cell to be "visible"
			double maxdistance,					//!< max distance for point to be visible from observer
			DRANGE hrzAngle,						//!< View angle horizontal limits (0 - 360 degrees)
			DRANGE vrtAngle,						//!< View angle vertical limits (-90 - 90 degrees)
			FLAGS flags = FLAG_None				//!< Optional flags: FLAG_EarthCurvature and FLAG_InvertTransparency
			);
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(OP::VIEWSHED::FLAGS)
#endif

} 	// end namespace OP
}	// end namespace RVC

#endif  // INC_RVC_OPVIEWSHED_H
