/**
 * \file rvc/rasttrnd.h 
 * \brief interface for the RVC::RASTERTREND class.
 *
 * \if NODOC
 * $Id: rasttrnd.h_v 1.11 2007/09/06 17:18:57 scowan Exp $
 *
 * $Log: rasttrnd.h_v $
 * Revision 1.11  2007/09/06 17:18:57  scowan
 * Read and write virtual methods changed to take objopenparms.
 *
 * Revision 1.10  2005/12/09 17:29:50  scowan
 * Updated to use credentials.
 *
 * Revision 1.9  2005/07/19 17:49:08  scowan
 * Update to use current simple methods.
 *
 * Revision 1.8  2005/01/21 23:15:43  scowan
 * Fixed deprecated method.
 *
 * Revision 1.7  2003/10/07 17:30:44  scowan
 * Object virtual method changes.
 *
 * Revision 1.6  2003/09/30 16:52:06  dwilliss
 * Doxygen
 *
 * Revision 1.5  2003/09/25 21:20:04  dwilliss
 * Fixed doxygen comment
 *
 * Revision 1.4  2003/09/15 13:49:45  fileserver!dwilliss
 * Doxygen
 *
 * Revision 1.3  2001/06/15 16:03:44  scowan
 * Added to RVC namespace.
 *
 * Revision 1.2  2000/10/19 21:14:29  scowan
 * Checked in for vacation.
 *
 * Revision 1.1  2000/09/29 19:44:26  scowan
 * Initial revision
 *
 * \endif
**/

//!	\class RVC::RASTERTREND rvc/rasttrnd.h
//!	\brief Manages RVC::OBJTYPE_RasterTrend objects.  
//!
//!	The interface currently wraps the MfRasterTrend functions.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_RasterTrend.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_RasterTrend.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_Raster
//!		- RVC::OBJTYPE_Hyperspectral
//!
//!	Read(), Make() and Write() methods inherited from RVC::SIMPLE class.  View RVC::SIMPLE docs for inherited methods

#ifndef INC_RVC_RASTTRND_H
#define INC_RVC_RASTTRND_H

#ifndef INC_RVC_SIMPLE_H
	#include <rvc/simple.h>
#endif

#ifndef INC_MI32_DOUBLEAR_H
	#include <mi32/doublear.h>
#endif

namespace RVC {

class RASTERTREND : public RVC::SIMPLE {
	public:
		RASTERTREND (
			);

		RASTERTREND (
			const RVC::RASTERTREND& rhs
			);

		virtual ~RASTERTREND (
			);

		RVC::RASTERTREND& operator= (
			const RVC::RASTERTREND& rhs
			);

		//! Obtain raster trend offset
		//! @return Raster trend offset
		double GetOffset (
			) const;

		//! Obtain raster trend order
		//! @return Raster trend order
		INT16 GetOrder (
			) const;

		//! Obtain raster trend scale
		//! @return Raster trend scale
		double GetScale (
			) const;

		//! Obtain raster trend list
		const DOUBLE_ARRAY<double>& GetTrendList (
			) const;

		//! Set raster trend offset
		void SetOffset (
			double Offset
			);

		//! Set raster trend order
		void SetOrder (
			INT16 order
			);

		//! Set raster trend scale
		void SetScale (
			double scale
			);

		//! Set raster trend list
		ERRVALUE SetTrendList (
			const DOUBLE_ARRAY<double>& TrendList
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Implementation of RVC::OBJECT virtual methods
		virtual OBJTYPE v_GetDftObjectType () const;

		//! Implementation of RVC::SIMPLE virtual methods
		virtual ERRVALUE v_GetDftNameDesc (const OBJECT& parent, DESCRIPTOR& Descriptor);
		virtual ERRVALUE v_MakeData (RVC::OBJMAKEPARMS& ObjMakeParms);
		virtual ERRVALUE v_ReadData (RVC::OBJOPENPARMS& ObjOpenParms);
		virtual ERRVALUE v_WriteData (RVC::OBJOPENPARMS& ObjOpenParms);

		double m_Offset;
		double m_Scale;
		INT16 m_Order;
		DOUBLE_ARRAY<double> m_TrendList;
		INT32 m_NumTrends;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

}		//! End of RVC namespace

#endif		//!< INC_RVC_RASTTRND_H
