/**
 * \file rvc/uiformdbrecord.h
 * \brief RVC::UI::FORM_DB_RECORD definitions
 *
 * \if NODOC
 * $Id: uiformdbrecord.h_v 1.1 2008/09/22 19:00:41 mju Exp $
 *
 * $Log: uiformdbrecord.h_v $
 * Revision 1.1  2008/09/22 19:00:41  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_UIFORMDBRECORD_H
#define	INC_RVC_UIFORMDBRECORD_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_RVC_DBTABLE_H
#include <rvc/dbtable.h>
#endif

#ifndef	INC_MI32_SERIALIZABLE_H
#include <mi32/serializable.h>
#endif

class DBEDIT_TINSTANCE;
namespace SML {
	union ARG;
	class CONTEXT;
	class STRING;
	}

namespace RVC {
namespace UI {
//=====================================================================================================================

//! Form for viewing and editing single database record.
class FORM_DB_RECORD : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style.
		enum FORMSTYLE {
			FORMSTYLE_Grid,
			FORMSTYLE_UserDefined
			};

		class SETTINGS : public SERIALIZABLE {
			public:
				SETTINGS ();
				SETTINGS (const SETTINGS& rhs);
				virtual ~SETTINGS ();
				SETTINGS& operator= (const SETTINGS& rhs);

				#ifndef GENERATING_DOXYGEN_OUTPUT
				class PRIV;
				SETTINGS& operator= (const PRIV& priv);
				const PRIV* GetPriv (
					) const { return ((this == 0) ? 0 : m_pPriv); }
			private:
				PRIV *m_pPriv;

				// SERIALIZABLE overrides.
				virtual ERRVALUE SerialRead (SERIALIZER& serializer);
				virtual ERRVALUE SerialWrite (SERIALIZER& serializer, const char *tagname) const;

				#endif
			};

		//! Install SML interface.
		static void SML_Install (
			SML::CONTEXT *context
			);

		//! Constructor.
		FORM_DB_RECORD (
			DBEDIT_TINSTANCE *idata,
			FORMSTYLE formstyle
			);

		//! Destructor.
		virtual ~FORM_DB_RECORD ();

		//! Clear layout settings "changed" status.
		void ClearChangedSettings ();

		//! Create form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			const SETTINGS *pSettings = 0			//!< Optional layout settings
			);

		//! Open dialog to edit user-defined form (MODE_UserDefined only).
		void DlgEditUserDefined ();

		//! Get current record reference type and number
		void GetRecordRef (
			DBTABLE::RECORDREFTYPE& RefType,		//!< Reference type returned
			INT32& RefNum								//!< Position number returned
			) const;

		//! Get current record reference number.
		INT32 GetRecordRefNum (
			) const;

		//! Get record reference type.
		DBTABLE::RECORDREFTYPE GetRecordRefType (
			) const;

		//! Get layout settings.
		void GetSettings (
			SETTINGS& settings						//!< Settings returned
			) const;

		//! Get current values.
		ERRVALUE GetValues (
			RVC::DBTABLE::RECORD& record			//!< Values returned
			);

		//! Determine if layout settings have changed.
		bool HasChangedSettings (
			) const;

		//! Determine if any field values have been changed since last saved.
		bool IsRecordChanged (
			) const;

		//! Notify form that containing dialog is being closed.
		//! Will hide, but not destroy, any open field value dialogs.
		void NotifyDlgClose ();

		//! Notify form that containing dialog was opened.
		//! Will show any existing field value dialogs.
		void NotifyDlgOpen ();

		//! Reread current record from table.
		ERRVALUE Reread ();

		//! Save record with optional prompt to user.
		//! @return EUserCancel if user chooses cancel, or error < 0, TRUE if saved, FALSE if not saved or no change.
		INT32 SaveRecord (
			bool AskUser = true						//!< Whether to prompt user to save or not
			);

		//! Set delegate for notifying when field value changed by user.
		void SetDelegateOnChangedFieldValue (
			DELEGATE_VOID_INT32 delegate
			);

		//! Set current record, will read values from table if existing.
		ERRVALUE SetRecordRef (
			DBTABLE::RECORDREFTYPE RefType,		//!< Reference type
			INT32 RefNum								//!< Reference number, -1 for RECORDREFTYPE_New record
			);

		//! Set values from record.
		ERRVALUE SetValues (
			const RVC::DBTABLE::RECORD& record,	//!< Values to set
			bool IsChanged								//!< Whether to treat as "changed" for saving
			);

		//! Undo all edits, restoring from saved record.
		void UndoAll ();

		//! Undo last edit.
		//! @return true if more edits to undo, false if not.
		bool UndoLast ();

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
	private:
		PRIV *m_pPriv;

		static void * SMLFUNC_Create (int, SML::ARG *args, UINT8 *, SML::CONTEXT *);
		static double SMLFUNC_GetFieldNum (int, SML::ARG *args, UINT8 *, SML::CONTEXT *);
		static SML::STRING* SMLFUNC_GetFieldStr (int, SML::ARG *args, UINT8 *, SML::CONTEXT *);
		static double SMLFUNC_SaveChanges (int, SML::ARG *args, UINT8 *, SML::CONTEXT *);
		static void SMLFUNC_SetField (int, SML::ARG *args, UINT8 *, SML::CONTEXT *);
		static void SMLFUNC_SetRecord (int, SML::ARG *args, UINT8 *, SML::CONTEXT *);

		FORM_DB_RECORD (const FORM_DB_RECORD&);
		FORM_DB_RECORD& operator= (const FORM_DB_RECORD&);
		#endif
	};

//=====================================================================================================================
}	// End namespace UI
}	// End namespace RVC


#endif	// INC_RVC_UIFORMDBRECORD_H
