/******************************************************************************
 *
 * \file sml/classarray.h
 * \brief SML CLASSARRAY
 *
 * \if NODOC
 * $Log: classarray.h_v $
 * Revision 1.3  2011/10/12 14:38:38  mju
 * Remove redundant hdr.
 *
 * Revision 1.2  2007/05/08 15:45:27  dwilliss
 * SML namespace
 *
 * Revision 1.1  2007/05/04 22:12:03  dwilliss
 * Initial revision
 *
 * \endif
*/
/*****************************************************************************/

#ifndef INC_SML_CLASSARRAY_H
#define INC_SML_CLASSARRAY_H

#ifndef	INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBEXPORT MI_DLLEXPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBEXPORT MI_DLLIMPORT
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

namespace SML {
class CLASS;
class CONTEXT;

//! Class used by SML to hold an array of class structures.
class RVCSYSLIBCLASSEXPORT CLASSARRAY {
	public:
		CLASSARRAY (
			CONTEXT* context,
			const CLASS* Class
			);

		virtual ~CLASSARRAY();

		INT32 GetNumItems() const;

		//! Get an item from the array
		void* GetItem(INT32 num);

		ERRVALUE Resize(INT32 NumItems);

		//! Set an item in the array. 
		//! Item number is 0-based.
		//! If bTakePossession is false, then the item's copy constructor will
		//!   be used to copy the class.  If it's true, then the array may just
		//!   keep the pointer passed in, and will free it when it's done with
		//!	it.  
		//! If attempting to set an item not already in the array, the array
		//! is expanded to be able to hold it.
		ERRVALUE SetItem(INT32 num, void* ptr, bool bTakePosession);

		const CLASS* GetClass() const;

#ifndef GENERATING_DOXYGEN_OUTPUT
	private:
		CONTEXT* m_context;
		const CLASS* m_Class;
		INT32 m_ItemSize;
		SIMPLE_ARRAY<void*> m_array;	// Only used if m_Class->size is 0
		SIMPLE_ARRAY<UINT8> m_buf;	// used for classes which have a "size" so 
											// that we don't allocate a whole lot of 
											// little structures (good for DPOINT2D, etc)

		// constructs a class instance in-place.
		// Only used for simple classes, not ones where we use new/delete
		void CallConstructor (
			void* data,
			const CLASS* Class
			);

		void* CallCopyConstructor (
			void* dest,
			void* source,
			const CLASS* Class
			);
#endif		// doxygen
	};


#undef RVCSYSLIBEXPORT
#undef RVCSYSLIBCLASSEXPORT


#if 0
// A work in progress - eventually I want SML::HASH to derive from this base
// class and then modify arrays to derive from it too.  That will simplify
// the parsing and virtual machine, plus improve how arrays are returned.
class RVCSYSLIBCLASSEXPORT CONTAINER {
	public:
		enum INDEXTYPE {
			INDEXTYPE_Integer = 0,
			INDEXTYPE_String = 1
			};

		CONTAINER (
			);
		virtual ~CONTAINER();

		const CLASS* GetClass() const;	// will return 0 if numeric

		int GetNumItems (
			) const;

		//! Returns the best index type.  Attempting to access elements in the
		//! container using the other index type will work, but is less effecient.
		INDEXTYPE GetBestIndexType (
			) const;

	private:
		virtual const LCLASS* v_GetClass() const = 0;	// will return 0 if numeric
		virtual int v_GetNumItems() const = 0;
		virtual bool v_Exists(const MISTRING&) const = 0;
		virtual void v_Delete(const MISTRING&) = 0;
		virtual MISTRINGLIST* v_GetKeyList() const = 0;
		virtual INDEXTYPE v_GetBestIndexType() const = 0;

	};
#endif


} // end of SML namespace

typedef SML::CLASSARRAY SMLCLASSARRAY;

#endif		//  INC_SML_CLASSARRAY_H


