/**
 * \file soa/arcimsuiconnection.h
 * \brief SOA ARCIMS User Interface for client Connection
 *
 * \if NODOC
 * $Id: arcimsuiconnection.h_v 1.4 2010/11/04 17:35:12 bcolombe Exp $
 *
 * $Log: arcimsuiconnection.h_v $
 * Revision 1.4  2010/11/04 17:35:12  bcolombe
 * *** empty log message ***
 *
 * Revision 1.3  2010/08/15 23:46:20  bcolombe
 * Added CreateSearchForm
 *
 * Revision 1.2  2010/08/11 21:52:10  bcolombe
 * Changes to make connection a form
 *
 * Revision 1.1  2007/09/14 16:57:09  dwilliss
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SOA_ARCIMSUICONNECTION_H
#define	INC_SOA_ARCIMSUICONNECTION_H

#ifndef	INC_MGUI_FORM_H
	#include <mgui/form.h>
#endif

#ifndef INC_SOA_UISERVICETREE_H
	#include <soa/uiservicetree.h>
#endif

class MISTRINGLIST;

namespace GRE{
class GROUP;
}

namespace SOA {
namespace ARCIMS {

class CONNECTION;

//=====================================================================================================================

class UI_FORM_CONNECTION_SMALL : public MGUI::FORM_COMPOSITE {
	public:

		//! Constructor.
		UI_FORM_CONNECTION_SMALL ();

		//! Destructor.
		virtual ~UI_FORM_CONNECTION_SMALL ();

		//! Create form.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Get current connection.
		const ARCIMS::CONNECTION& GetConnection (
			) const;

		//! Set delegate to call when connection changed.
		void SetDelegateOnChange (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set connection.
		ERRVALUE SetConnection (
			const ARCIMS::CONNECTION& Connection,
			bool notify
			);

		//! Set server URL.
		ERRVALUE SetServerURL (
			const MISTRING& ServerURL,
			bool notify
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		UI_FORM_CONNECTION_SMALL (const UI_FORM_CONNECTION_SMALL&);
		UI_FORM_CONNECTION_SMALL& operator= (const UI_FORM_CONNECTION_SMALL&);
		#endif
	};

class UI_FORM_CONNECTION : public MGUI::FORM_COMPOSITE {
	public:

		UI_FORM_CONNECTION (
			bool bShowPreview = false,
			const GRE::GROUP* group=0
			);

		UI_FORM_CONNECTION (
			const ARCIMS::CONNECTION& connection,
			const GRE::GROUP* group=0
			);

		~UI_FORM_CONNECTION ();

		//! Create form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Create only the search form.
		ERRVALUE CreateSearchForm (
			MGUI::LAYOUT_PANE_BASE& ParentPane
			);

		//! Get current connection.
		const ARCIMS::CONNECTION& GetConnection (
			) const;

		//! Get default layers
		const MISTRINGLIST GetDefaultLayers (
			) const;

		//! Has item selected
		bool HasConnection (
			) const;

		void OnOpen();

		void OnOK();
		
		//! Set connection.
		void SetConnection (
			const ARCIMS::CONNECTION& connection
			);

		//! delegate when user clicks on item
		void SetDelegateOnSelectService (
			SOA::CTRL_SERVICETREE::DELEGATE_SERVERLISTITEM delegate
			);

		//! delegate when user double clicks on item
		void SetDelegateOnDoubleClickService (
			SOA::CTRL_SERVICETREE::DELEGATE_SERVERLISTITEM delegate
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		UI_FORM_CONNECTION (const UI_FORM_CONNECTION&);
		UI_FORM_CONNECTION& operator= (const UI_FORM_CONNECTION&);
		#endif
	};

//=====================================================================================================================

//! Dialog to prompt user to choose ARCIMS.
//! @return TRUE if connection changed, FALSE if not, < 0 if error or Cancel
int DlgGetConnection (
	MDLGPARENT dlgparent,							//!< Parent for dialog
	ARCIMS::CONNECTION& connection,					//!< Connection passed/returned
	const GRE::GROUP *group=0,						//!< Group to Match the bounding box to when searching for arcims layers from catalog
	MISTRINGLIST *defaultlayers=0					//!< Layer name that the user selected in the dialog
	);

//=====================================================================================================================

}	// End namespace ARCIMS
}	// End namespace SOA

#endif	// INC_SOA_WMSUICONNECTION_H
