/**
 * \file imagefilterextend.h <spatmod/imagefilterextend.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_EXTEND definition.
 *
 * \if NODOC
 * $Id: imagefilterextend.h_v 1.2 2008/07/01 20:26:52 mju Exp $
 *
 * $Log: imagefilterextend.h_v $
 * Revision 1.2  2008/07/01 20:26:52  mju
 * Add v_queryIterator override.
 *
 * Revision 1.1  2007/03/21 20:27:12  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTEREXTEND_H
#define	INC_SPATMOD_IMAGEFILTEREXTEND_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to "extend" image by adding null pixels around outside.
//! Will "crop" image if necessary if result does not entirely contain input image.
class FILTER_EXTEND : public FILTER {
	public:

		//! Construct with specified rectangle.
		//! Resulting image must overlap source by at least one pixel or Initialize will return error.
		FILTER_EXTEND (
			STAGE *pInputImage,						//!< Input image stage
			bool TakeOwnership,						//!< Filter takes ownership of source stage and will delete when needed
			INT32 NumColumnsAdd,						//!< Number of columns to add at "left" of source stage, can be negative
			INT32 NumRowsAdd,							//!< Number of rows to add at "top" of source image, can be negative
			UINT32 NewTotalColumns = 0,			//!< New total number of columns, 0 to not extend at "right" or source
			UINT32 NewTotalRows = 0					//!< New total number of rows, 0 to not extend at "bottom" or source
			);

		// Destructor.
		virtual ~FILTER_EXTEND ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_NumColumnsAdd;
		INT32 m_NumRowsAdd;
		UINT32 m_NewTotalColumns;
		UINT32 m_NewTotalRows;

		ERRVALUE BeginIteratorPrevStage (
			const SECTION_ITERATOR& Iterator,
			SECTION_ITERATOR& IteratorPrev
			) const;

		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR& iterator) const;

		// Unimplemented
		FILTER_EXTEND (const FILTER_EXTEND&);
		FILTER_EXTEND& operator= (const FILTER_EXTEND&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTEREXTEND_H

