/**
 * \file imagefiltermultiply.h <spatmod/imagefiltermultiply.h>
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_MULTIPLY definition.
 *
 * \if NODOC
 * $Id: imagefiltermultiply.h_v 1.1 2006/05/04 16:50:01 mju Exp $
 *
 * $Log: imagefiltermultiply.h_v $
 * Revision 1.1  2006/05/04 16:50:01  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERMULTIPLY_H
#define	INC_SPATMOD_IMAGEFILTERMULTIPLY_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class BUFFER;

//=====================================================================================================================

//! Filter to multiply images.
//! Allows multiplication of multiple components or combination of single and multi-component images.
//! All multi-component stages must have same number of components.
//! Single-component stages will be multiplied by all components of multi-component stages.
class FILTER_MULTIPLY : public FILTER {
	public:

		//! Construct using array of previous stage pointers and common ownership.
		//! All previous stages must have same total dimensions.
		//! If no pixel properties are specified, the resulting sample data types will be determined as the
		//! minimum size needed to prevent overflow.  If pixel properties are specified, no range check will
		//! be performed, so overflow may occur if sample datatype range is too small.
		FILTER_MULTIPLY (
			STAGE_ARRAY& PreviousStages,			//!< Array of previous stages
			bool TakeOwnership,						//!< Filter takes ownership of previous stages and will delete when needed
			const PIXEL *pPixelProperties = 0	//!< Resulting pixel properties, 0 to automatically determine
			);

		// Destructor.
		virtual ~FILTER_MULTIPLY ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		// STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION& SubSection, BUFFER& buffer);

		// Unimplemented
		FILTER_MULTIPLY (const FILTER_MULTIPLY&);
		FILTER_MULTIPLY& operator= (const FILTER_MULTIPLY&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERMULTIPLY_H

