/**
 * \file spatmod/imagetargettilesetmicrosoftbing.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::TARGET_TILESET_MICROSOFTBING definition.
 *
 * \if NODOC
 * $Id: imagetargettilesetmicrosoftbing.h_v 1.15 2011/10/11 15:03:38 mju Exp $
 *
 * $Log: imagetargettilesetmicrosoftbing.h_v $
 * Revision 1.15  2011/10/11 15:03:38  mju
 * Remove textid hdr.
 *
 * Revision 1.14  2011/04/20 23:20:02  bcolombe
 * Add ResampleMethod for reduced levels
 *
 * Revision 1.13  2011/04/14 16:43:23  bcolombe
 * Remove extents from ctor, determine from input
 *
 * Revision 1.12  2010/11/01 22:45:42  bcolombe
 * Imgformat changes
 *
 * Revision 1.11  2010/06/25 16:32:37  bcolombe
 * Add UsePalette to ComputeReduced
 *
 * Revision 1.10  2010/05/18 16:51:44  bcolombe
 * Add ComputeReducedLevel that takes source & target locations
 *
 * Revision 1.9  2010/04/20 22:50:53  bcolombe
 * Add zipoutput to ComputeReducedLevel
 *
 * Revision 1.8  2010/04/19 16:32:12  bcolombe
 * Added bOverwriteExisting to CreateHTML
 *
 * Revision 1.7  2010/04/02 22:26:17  bcolombe
 * Make CreateHTML and ComputeReducedLevel static
 *
 * Revision 1.6  2010/03/31 22:38:13  bcolombe
 * Added MinLevel to CreateHTML
 *
 * Revision 1.5  2010/03/18 16:04:23  bcolombe
 * Change to constructor
 *
 * Revision 1.4  2009/12/03 21:56:12  bcolombe
 * Added optional message to ComputeReducedLevel
 *
 * Revision 1.3  2009/11/04 23:47:00  bcolombe
 * Constructor takes basepath instead of htmlpath
 *
 * Revision 1.2  2009/11/04 21:30:33  bcolombe
 * Changed description of filepath in constructor
 *
 * Revision 1.1  2009/08/28 08:31:39  bcolombe
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGETARGETTILESETMICROSOFTBING_H
#define	INC_SPATMOD_IMAGETARGETTILESETMICROSOFTBING_H

#ifndef	INC_SPATMOD_IMAGETARGETTILESETFILE_H
#include <spatmod/imagetargettilesetfile.h>
#endif

class MISTATUSCONTEXT;

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Google Maps target.
class TARGET_TILESET_MICROSOFTBING : public TARGET_TILESET_FILE {
	public:

		//! Create Bing HTML file
		static ERRVALUE CreateHTML (
			TILESET& Tileset,								//!< Tileset to use
			bool bOverwriteExisting = true,					//!< Write over any existing files
			INT32 MinLevel = -1								//!< Minimum zoom level to include
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			INT32 level,										//!< Level to compute
			MISTRING message = TEXTID_Creating_ResolutionLevelN	//!< Context message to display
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			const FILEPATH& SourcePath,							//!< Location of source tiles
			const FILEPATH& TargetPath,							//!< Location to output tiles
			INT32 level,										//!< Level to compute
			MISTRING message = TEXTID_Creating_ResolutionLevelN	//!< Context message to display
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			INT32 level,										//!< Level to compute
			TILESET::IMGFORMAT ImgFormat = TILESET::IMGFORMAT_Invalid,	//!< ImgFormat
			UINT16 QualityJPEG = 75,							//!< Quality of JPEG output
			MISTRING message = TEXTID_Creating_ResolutionLevelN,//!< Context message to display
			bool bZipOutput = false								//!< Zip results
			);

		//! Compute reduced level (tiles must exist at zoom level + 1)
		static ERRVALUE ComputeReducedLevel (
			TILESET& Tileset,									//!< Tileset to use
			const FILEPATH& SourcePath,							//!< Location of source tiles
			const FILEPATH& TargetPath,							//!< Location to output tiles
			INT32 level,										//!< Level to compute
			TILESET::IMGFORMAT ImgFormat,						//!< ImgFormat
			const IMAGE::PIXEL& PixelProperties,				//!< Pixel Properties
			UINT16 QualityJPEG = 75,							//!< Quality of JPEG output
			MISTRING message = TEXTID_Creating_ResolutionLevelN,//!< Context message to display
			bool bZipOutput = false,								//!< Zip results
			RESAMPLEMETHOD ResampleMethod = RESAMPLEMETHOD_Nearest	//!< Resample Method to use for zoom filter
			);

		//! Construct target.
		TARGET_TILESET_MICROSOFTBING (
			STAGE *pInputStage,					//!< Stage in pipeline to obtain image data from
			TILESET::IMGFORMAT format,					//!< Image tile file format
			const FILEPATH& filepath,			//!< Basepath used for creating TSD and HTML files
			INT32 TileSize,						//!< Tile Size (256 or 128)
			INT32 MinLevel,						//!< Minimum Zoom Level
			INT32 MaxLevel						//!< Maximum Zoom Level
			);

		//! Destructor.
		virtual ~TARGET_TILESET_MICROSOFTBING ();
		
		//! Set parameters specific to JPEG format.
		void SetParmsJPEG (
			INT16 quality
			) { m_QualityJPEG = quality; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		class IMAGEMICROSOFTBING;

		FILEPATH m_basepath;
		UINT16 m_QualityJPEG;
		MISTATUSCONTEXT *m_pStatusContext;

		// TARGET overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PushBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PushEnd ();

		// TARGET_TILESET_FILE overrides.
		ERRVALUE v_CreateTargetFile (IMAGE::STAGE *pStageInput, const FILEPATH& filepath, const IMAGE::FORMAT& format, IMAGE::TARGET_FILE *& pTargetFile) const;
		const IMAGE::FORMAT* v_FindTileFile (FILEPATH& filepath, const LPOINT3D& TileIdx) const;
		ERRVALUE v_GenerateTileFilePath (FILEPATH& filepath, const LPOINT3D& TileIdx, bool HasNullCells, bool CreateFolder) const;

		// Unimplemented
		TARGET_TILESET_MICROSOFTBING (const TARGET_TILESET_MICROSOFTBING&);
		TARGET_TILESET_MICROSOFTBING& operator= (const TARGET_TILESET_MICROSOFTBING&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGETARGETTILESETMICROSOFTBING_H

